package com.emato.ccnet.wx.biz.req.order.csp;

import com.emato.ccnet.wx.biz.CebCustomsResponse;
import com.emato.ccnet.wx.contants.Contants;
import com.emato.ccnet.wx.dao.entity.ccnet.CusOrder;
import com.emato.ccnet.wx.dao.entity.ccnet.CusOrderResp;
import com.emato.ccnet.wx.dao.entity.ccnet.CusOrderStatus;
import com.emato.ccnet.wx.dto.ccnetdto.*;
import com.emato.ccnet.wx.manager.result.Result;
import com.emato.ccnet.wx.service.*;
import com.emato.ccnet.wx.util.jackson.JacksonUtil;
import com.google.common.collect.ImmutableMap;
import com.google.gson.Gson;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.Map;

@Component
public class Ceb311CustiomsResponse implements CebCustomsResponse {
    private static final Logger LOGGER = LoggerFactory.getLogger(Ceb311CustiomsResponse.class);

    @Autowired
    private CusOrderRespService cusOrderRespService;
    @Autowired
    private CusOrderStatusService cusOrderStatusService;
    @Autowired
    private CusOrderService cusOrderService;
    @Autowired
    private WxOrderService wxOrderService;
    @Autowired
    private WxOrderGrabService wxOrderGrabService;

    @Override
    public int biz(Result result) {
        //解析数据
        OrderCustiomsResponse orderCustiomsResponse = analysisCustiomsResponse(result);
        if(orderCustiomsResponse == null){
            LOGGER.info("解析响应数据失败");
            return 0;
        }
        OrderDto orderDto = orderCustiomsResponse.getClsOrderDto();
        if(orderDto == null){
            LOGGER.info("csp返回订单数据获取失败");
            return 1;
        }
        String orderSn = orderDto.getOrderSn();
        String orderNo = orderDto.getOrderNo();

        //查找电子订单
        CusOrder cusOrder = cusOrderService.get(orderSn);
        if(cusOrder == null){
            LOGGER.info("查找电子订单表数据失败");
            return 1;
        }

        //将电子订单回执初始化
        CusOrderResp cusOrderResp = cusOrderRespService.insertCusOrderRespByCustioms(orderCustiomsResponse,cusOrder.getCusOrderSn());
        if(cusOrderResp == null){
            LOGGER.info("电子订单海关回执初始化数据失败");
            return 1;
        }

        //查找电子订单状态表
        Map<String, Object> reqParam = ImmutableMap.of("orderSn", cusOrderResp.getOrderSn(),"sendStatus", Contants.DECLARATION_SUCCESS_STATUS);
        CusOrderStatus cusOrderStatus = cusOrderStatusService.findCusOrderStatus(reqParam);
        if(cusOrderStatus == null){
            LOGGER.info("查找电子订单状态表数据失败");
            return 1;
        }

        //对响应的数据进行处理
        if(Contants.ORDER_RESPONSE_SUCCESS.equals(orderCustiomsResponse.getCode())){
            cusOrderStatus.setSendStatus(Contants.SUCCESS_STATUS);
            Map<String, Object> wxMap = ImmutableMap.of("orderSn", orderNo, "clecOrderStatus", Contants.SUCCESS_STATUS);
            wxOrderService.updateWxOrderStatus(wxMap);
            Map<String, Object> recordMap = ImmutableMap.of("orderSn", orderNo, "eleOrderSuccTime", new Date(),"isEleOrderSend", Contants.WX_SUCC);
            wxOrderGrabService.updateMallOrderProcessRecord(recordMap);
        }

        if(Contants.ORDER_RESPONSE_FAIL.equals(orderCustiomsResponse.getCode())){
            cusOrderStatus.setSendStatus(Contants.FAIL_STATUS);
            Map<String, Object> wxMap = ImmutableMap.of("orderSn", orderNo, "clecOrderStatus", Contants.FAIL_STATUS);
            wxOrderService.updateWxOrderStatus(wxMap);
            Map<String, Object> recordMap = ImmutableMap.of("orderSn", orderNo, "eleOrderSuccTime", new Date(),"isEleOrderSend", Contants.WX_FAIL);
            wxOrderGrabService.updateMallOrderProcessRecord(recordMap);

        }

        //对回调报文进行处理:平台回执
        if(orderCustiomsResponse.getEportResultStatusDto() != null){
            EportResultStatusDto eportResultStatusDto = orderCustiomsResponse.getEportResultStatusDto();
            if(Contants.ORDER_PLAT_SUCCESS.equals(eportResultStatusDto.getCode())){
                cusOrderStatus.setPlatRespStatus(Contants.SUCCESS_STATUS);
            }
            if(Contants.ORDER_PLAT_FAIL.equals(eportResultStatusDto.getCode())){
                cusOrderStatus.setPlatRespStatus(Contants.FAIL_STATUS);
            }
        }

        //海关回执
        if(orderCustiomsResponse.getCusResultStatusDto() != null){
            CusResultStatusDto cusResultStatusDto = orderCustiomsResponse.getCusResultStatusDto();
            if(Contants.CUS_DECLARATION.equals(cusResultStatusDto.getCode())){
                cusOrderStatus.setCusRespStatus(Contants.SUCCESS_STATUS);
            }
            if(Contants.CUS_CHARGEBACK.equals(cusResultStatusDto.getCode()) || Contants.CUS_FAIL.equals(cusResultStatusDto.getCode())){
                cusOrderStatus.setCusRespStatus(Contants.FAIL_STATUS);
                cusOrder.setAppType(Contants.APP_STATUS_SEPARETE);
            }
        }

        //国检回执
        if(orderCustiomsResponse.getCiqResultStatusDto() != null){
            CiqResultStatusDto ciqResultStatusDto = orderCustiomsResponse.getCiqResultStatusDto();
            if(Contants.CIQ_SUCCESS.equals(ciqResultStatusDto.getCode())){
                cusOrderStatus.setCipRespStatus(Contants.SUCCESS_STATUS);
            }
            if(Contants.CIQ_FAIL.equals(ciqResultStatusDto.getCode())|| Contants.CIQ_RECEIVE_FAIL.equals(ciqResultStatusDto.getCode())){
                cusOrderStatus.setCipRespStatus(Contants.FAIL_STATUS);
                cusOrder.setAppType(Contants.APP_STATUS_SEPARETE);
            }
        }

        int cusOrderSta = cusOrderStatusService.updateCusOrderStatus(cusOrderStatus);
        if(cusOrderSta != 1){
            LOGGER.info("电子订单海关回执更新数据失败");
            return 1;
        }

        CusOrder CusOrder = cusOrderService.update(cusOrder);
        if(CusOrder == null){
            LOGGER.info("电子订单海关回执更新数据失败");
            return 1;
        }

        LOGGER.info("海关处理回调成功");
        return 2;
    }

    /**
     * 解析海关回调数据
     * @param response
     * @return
     */
    private OrderCustiomsResponse analysisCustiomsResponse(Result response) {
        Gson gson = new Gson();
        if(response == null){
            LOGGER.info("解析响应数据失败");
            return null;
        }

        //解析回执数据
        String data = JacksonUtil.toJson(response.getData());
        LOGGER.info("海关回执 entity:"+data);

        List<OrderCustiomsResponse> list = gson.fromJson(data, List.class);
        if(list == null && list.size() <= 0){
            LOGGER.info("解析响应数据失败");
            return null;
        }

        return gson.fromJson(gson.toJson(list.get(0)), OrderCustiomsResponse.class);
    }
}
