package com.emato.ccnet.wx.biz.req.order.csp;

import com.emato.ccnet.wx.biz.CebReqBiz;
import com.emato.ccnet.wx.contants.CcnetContants;
import com.emato.ccnet.wx.contants.Contants;
import com.emato.ccnet.wx.dao.entity.ccnet.WxOrder;
import com.emato.ccnet.wx.dto.ccnetdto.OrderBill;
import com.emato.ccnet.wx.service.CusOrderStatusService;
import com.emato.ccnet.wx.service.WxOrderService;
import com.emato.ccnet.wx.support.msg.resp.ResponseMessage;
import com.google.common.collect.ImmutableMap;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

@Component
public class Ceb311OrderBiz implements CebReqBiz {
    private static final Logger LOGGER = LoggerFactory.getLogger(Ceb311OrderBiz.class);
    private Map<String, Object> param = Collections.EMPTY_MAP;

    @Autowired
    private Ceb311OrderGetDate ceb311OrderGetDate;

    @Autowired
    private Ceb311OrderAddDate ceb311OrderAddDate;

    @Autowired
    private Ceb311SendDateRequet ceb311SendDateRequet;

    @Autowired
    private CusOrderStatusService cusOrderStatusService;

    @Autowired
    private WxOrderService wxOrderService;

    @Value("${db.ccnet-send-wm.page-size}")
    private String limit = "20";
    /**
     * 电子订单正常业务入口
     * @return
     */
    @Override
    public ResponseMessage biz() {

        Map<String,Object> reqParam = ImmutableMap.of("number", Integer.valueOf(limit),"wxOrderDetail", CcnetContants.isscan.item_1.getItem(),"clecOrderStatus", CcnetContants.sendStatus.item_0.getItem(),"buyerPayCheck",CcnetContants.isscan.item_1.getItem());
        //查询中网电子订单数据
        List list = ceb311OrderGetDate.getData(reqParam);
        if (list.isEmpty()) {
            String info = "【CEB311】没有订单数据, 生成海关支付单报文提前结束";
            LOGGER.error("--- {}", info);
            afterFailed(list);
            return ResponseMessage.failed(info);
        }

        //添加erp电子订单数据
        ResponseMessage responseMessage = ceb311OrderAddDate.addDate(list);
        if(!ResponseMessage.isSuccess(responseMessage)){
            String info = "生成海关支付单初始化失败" + responseMessage.getMsg();
            LOGGER.error("--- {}", info);
            afterFailed(list);
            return ResponseMessage.failed(info);
        }

        //发送数据csp系统请求并响应更新订单
        ResponseMessage ceb311ResponseMessage = ceb311SendDateRequet.sendDate();
        if(!ResponseMessage.isSuccess(ceb311ResponseMessage)){
            String info = "发送请求响应失败"+ceb311ResponseMessage.getMsg();
            LOGGER.error("--- {}", ceb311ResponseMessage.getMsg());
            afterFailed(list);
            return ResponseMessage.failed(info);
        }

        return ResponseMessage.success();
    }

    private void afterFailed(List list) {
        List<String> failedList = new ArrayList<>();
        List<Map<String, Object>> wmFaileds = new ArrayList<>();

        list.stream().forEach(x->{
            failedList.add(((WxOrder)x).getOrderSn());
        });
        if (failedList.size() > 0) {
            failedList.stream().forEach(x->{
                Map<String, Object> wmMap = ImmutableMap.of("orderSn", x, "clecOrderStatus", Contants.FAIL_STATUS);
                wmFaileds.add(wmMap);
            });
            if (wmFaileds.size() > 0) {
                wxOrderService.updateWxOrderStatusBatch(wmFaileds);
            }
        }
    }

    public Map<String, Object> getParam() {
        return param;
    }

    public void setParam(Map<String, Object> param) {
        this.param = param;
    }

}
