package com.emato.ccnet.wx.biz.req.order.csp;

import com.emato.ccnet.wx.biz.CebReqSendDateAfter;
import com.emato.ccnet.wx.contants.Contants;
import com.emato.ccnet.wx.service.CusOrderStatusService;
import com.emato.ccnet.wx.service.WxOrderService;
import com.google.common.collect.ImmutableMap;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Component
public class Ceb311SendDateAfter implements CebReqSendDateAfter{
    private static final Logger LOGGER = LoggerFactory.getLogger(Ceb311SendDateAfter.class);

    @Autowired
    private CusOrderStatusService cusOrderStatusService;

    @Autowired
    private WxOrderService wxOrderService;

    /**
     * 状态修改
     * @param sucList
     * @param failedList
     * @return
     */
    @Override
    public boolean afterSend(List<String> sucList, List<String> failedList) {
        List<Map<String, Object>> sucs = new ArrayList<>();
        List<Map<String, Object>> wxSucs = new ArrayList<>();
        List<Map<String, Object>> faileds = new ArrayList<>();
        List<Map<String, Object>> wxFaileds = new ArrayList<>();
        //全部处理完后
        if (sucList.size() > 0) {
            LOGGER.info("--- 【CEB311】更新【成功】报文订单记录......");
            sucList.stream().forEach(x->{
                Map<String, Object> map = ImmutableMap.of("orderSn", x, "sendStatus", Contants.DECLARATION_SUCCESS_STATUS);
                Map<String, Object> wmMap = ImmutableMap.of("orderSn", x, "clecOrderStatus", Contants.DECLARATION_STATUS);
                sucs.add(map);
                wxSucs.add(wmMap);
            });
            if(sucs.size() > 0){
                cusOrderStatusService.updateCusOrderStatusBatch(sucs);
            }
            //kmall中的支付单修改为失败
            if (wxSucs.size() > 0) {
                wxOrderService.updateWxOrderStatusBatch(wxSucs);
            }
        }

        if (failedList.size() > 0) {
            LOGGER.info("--- 【CEB311】更新【失败】报文订单记录......");
            failedList.stream().forEach(x->{
                Map<String, Object> map = ImmutableMap.of("orderSn", x, "sendStatus", Contants.FAIL_STATUS);
                Map<String, Object> wmMap = ImmutableMap.of("orderSn", x, "clecOrderStatus", Contants.FAIL_STATUS);
                faileds.add(map);
                wxFaileds.add(wmMap);
            });
            //ccnet中的支付订单修改为失败
            if (faileds.size() > 0) {
                cusOrderStatusService.updateCusOrderStatusBatch(faileds);
            }
            //kmall中的支付单修改为失败
            if (wxFaileds.size() > 0) {
                wxOrderService.updateWxOrderStatusBatch(wxFaileds);
            }
        }
        return true;
    }
}
