package com.emato.ccnet.wx.biz.req.order.csp;

import com.emato.ccnet.wx.biz.CebReqSendDate;
import com.emato.ccnet.wx.config.Cus;
import com.emato.ccnet.wx.contants.Contants;
import com.emato.ccnet.wx.dao.entity.ccnet.CusOrder;
import com.emato.ccnet.wx.dao.entity.ccnet.CusOrderGoods;
import com.emato.ccnet.wx.dao.entity.ccnet.CusOrderStatus;
import com.emato.ccnet.wx.dto.ccnetdto.OrderDto;
import com.emato.ccnet.wx.dto.ccnetdto.OrderRequest;
import com.emato.ccnet.wx.manager.result.Result;
import com.emato.ccnet.wx.service.CusOrderGoodsService;
import com.emato.ccnet.wx.service.CusOrderService;
import com.emato.ccnet.wx.service.CusOrderStatusService;
import com.emato.ccnet.wx.support.msg.resp.ResponseMessage;
import com.emato.ccnet.wx.util.OkHttpUtils;
import com.emato.ccnet.wx.util.jackson.JacksonUtil;
import com.google.common.base.Joiner;
import com.google.common.collect.ImmutableMap;
import com.google.common.collect.Lists;
import okhttp3.Request;
import okhttp3.RequestBody;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.util.*;

@Component
public class Ceb311SendDateRequet implements CebReqSendDate {
    private static final Logger LOGGER = LoggerFactory.getLogger(Ceb311SendDateRequet.class);

    @Autowired
    private CusOrderService cusOrderService;

    @Autowired
    private Ceb311SendDateResponse ceb311SendDateResponse;

    @Autowired
    private CusOrderGoodsService cusOrderGoodsService;

    @Autowired
    private Ceb311SendDateAfter ceb311SendDateAfter;

    @Autowired
    private CusOrderStatusService cusOrderStatusService;

    @Autowired
    private Cus cus;

    /**
     * 发送请求
     * @return
     */
    @Override
    public ResponseMessage sendDate() {
        //查找当前ccnet中的所有电子订单
        List<CusOrder> cusOrders = cusOrderService.findAllCusOrder();
        if(cusOrders == null){
            String info = "查找当前ccnet中的所有电子订单失败";
            LOGGER.error("--- {}", info);
            return ResponseMessage.failed(info);
        }

        //发送给第三方的请求回调
        ResponseMessage orderSumbit = buildRequest(cusOrders);
        if(!ResponseMessage.isSuccess(orderSumbit)){
            String info = "查找当前ccnet中的所有电子订单失败";
            LOGGER.error("--- {}", info);
            return ResponseMessage.failed(info);
        }
        return ResponseMessage.success();
    }

    /**
     * 发送第三方请求和回调
     * @param cusOrders
     * @return
     */
    private ResponseMessage buildRequest(List<CusOrder> cusOrders) {

        List<String> sucList = new ArrayList<>();
        List<String> failedList = new ArrayList<>();
        for (CusOrder cusOrder : cusOrders){

            //更新电子订单数据为申报中
            ResponseMessage updateCusOrderStatus = updateCusOrderStatus(cusOrder.getOrderSn());
            if(!ResponseMessage.isSuccess(updateCusOrderStatus)){
                failedList.add(cusOrder.getOrderNo());
                String info = "更新电子订单数据为申报中失败：" + updateCusOrderStatus.getMsg();
                LOGGER.error("--- {}", info);
                continue;
            }

            //数据组装
            Map<String, String> sPara = assembleData(cusOrder);
            if(sPara == null){
                failedList.add(cusOrder.getOrderNo());
                String info = "数据组装失败";
                LOGGER.error("--- {}", info);
                continue;
            }

            //发送CSP系统请求并获取响应信息
            String response = requestSubmit(sPara);
            LOGGER.info("CSP的响应报文：" + response);
            if(response == null){
                failedList.add(cusOrder.getOrderNo());
                String info = "发送请求并获取响应信息失败";
                LOGGER.error("--- {}", info);
                continue;
            }

            LOGGER.info("response date:"+response);

            //解析响应报文并处理本地业务逻辑
            ResponseMessage responseDis = ceb311SendDateResponse.addResponse(response,cusOrder.getOrderSn());
            if(!ResponseMessage.isSuccess(responseDis)){
                failedList.add(cusOrder.getOrderNo());
                String info = "解析响应报文并处理本地业务逻辑失败";
                LOGGER.error("--- {}", info);
                continue;
            }

            sucList.add(cusOrder.getOrderNo());
        }

        LOGGER.info("--- 【CEB311】【成功】订单数:{}, 订单: {}", sucList.size(), Joiner.on(",").skipNulls().join(sucList.toArray()));
        LOGGER.info("--- 【CEB311】【失败】订单数:{}, 订单: {}", failedList.size(), Joiner.on(",").skipNulls().join(failedList.toArray()));

        ceb311SendDateAfter.afterSend(sucList,failedList);

        return ResponseMessage.success();
    }
    /**
     * 发送请求并获取响应信息
     * @param sPara
     * @return
     */
    private String requestSubmit(Map<String, String> sPara) {
        //构造RequestBody
        RequestBody requestBody = OkHttpUtils.buildRequestBody(sPara);

        //构建Request
        String url = cus.getOrder().get("order-url");
        Request request = OkHttpUtils.buildRequest(url,requestBody);

        // 同步访问,返回结果字符串
        String response = null;
        try {
            response = OkHttpUtils.post(request);
        } catch (IOException e) {
            e.printStackTrace();
        }

        return response;
    }

    /**
     * 更新电子订单状态表
     * @param orderSn
     * @return
     */
    private ResponseMessage updateCusOrderStatus(String orderSn) {
        //通过编号查找电子订单表
        Map<String, Object> reqParam = ImmutableMap.of("orderSn", orderSn,"sendStatus", Contants.INIT_STATUS);
        CusOrderStatus cusOrderStatus = cusOrderStatusService.findCusOrderStatus(reqParam);
        if(cusOrderStatus == null){
            return ResponseMessage.failed("该电子订单不存在");
        }

        //更新请求报文状态:请求申报中
        cusOrderStatus.setModerSn("");
        cusOrderStatus.setModTime(new Date());
        cusOrderStatus.setSendStatus(Contants.DECLARATION_STATUS);
        int cusOrderSta = cusOrderStatusService.updateCusOrderStatus(cusOrderStatus);
        if(cusOrderSta != 1){
            return ResponseMessage.failed("该电子订单更新失败");
        }
        return ResponseMessage.success();
    }

    /**
     * 组装json中所要的数据
     * @param cusOrder
     * @return
     */
    private Map<String, String> assembleData(CusOrder cusOrder) {
        // 发送电子订单的集合
        Map<String, String> sPara = new HashMap<>();
        OrderDto clsOrderDto = new OrderDto(cusOrder);

        //通过电子订单内部编号查找海关电子订单商品
        List<CusOrderGoods> cusOrderGoodes = cusOrderGoodsService.findCusOrderGoodsByCusOrderSn(cusOrder.getCusOrderSn());
        if (cusOrderGoodes == null || cusOrderGoodes.size() <= 0) {
            LOGGER.error("通过电子订单内部编号查找海关电子订单商品失败，电子订单内部编号："+cusOrder.getCusOrderSn());
            return null;
        }
        for (CusOrderGoods orderGoods:cusOrderGoodes) {
            if(orderGoods.getBrand()== null || "".equalsIgnoreCase(orderGoods.getBrand())){
                orderGoods.setBrand("无");
            }
        }
        OrderRequest request = new OrderRequest();
        request.setClsOrderDetailDto(cusOrderGoodes);
        request.setClsOrderDto(clsOrderDto);
        String cusOrderGoodesJson = JacksonUtil.toJson(request);
        sPara.put("data", cusOrderGoodesJson);
        LOGGER.info("request Data :" + cusOrderGoodesJson);
        return sPara;
    }

}
