package com.emato.ccnet.wx.biz.req.play.wx;

import com.emato.ccnet.wx.biz.CebCustomsResponse;
import com.emato.ccnet.wx.contants.CcnetContants;
import com.emato.ccnet.wx.contants.Contants;
import com.emato.ccnet.wx.contants.SysContants;
import com.emato.ccnet.wx.contants.WmContants;
import com.emato.ccnet.wx.dao.entity.ccnet.WxCuspayResp;
import com.emato.ccnet.wx.dao.entity.ccnet.WxCuspayStatus;
import com.emato.ccnet.wx.dao.entity.ccnet.WxOrder;
import com.emato.ccnet.wx.dao.entity.ccnet.WxPayDoc;
import com.emato.ccnet.wx.dto.ccnetdto.PlayResponse;
import com.emato.ccnet.wx.manager.result.Result;
import com.emato.ccnet.wx.manager.snow.SnowflakeUtil;
import com.emato.ccnet.wx.service.*;
import com.emato.ccnet.wx.util.MapBeanUtil;
import com.google.common.collect.ImmutableMap;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.Map;

@Component
public class WxplayResponse  implements CebCustomsResponse {
    private static final Logger LOGGER = LoggerFactory.getLogger(WxplayResponse.class);

    @Autowired
    private WxCuspayStatusService wxCuspayStatusService;
    @Autowired
    private WxCuspayRespService wxCuspayRespService;
    @Autowired
    private WxPayDocService wxPayDocService;

    @Autowired
    private WxOrderService wxOrderService;

    @Autowired
    private WxOrderGrabService wxOrderGrabService;

    @Override
    public int biz(Result result) {
        //解析数据
        PlayResponse playResponse = analysisResponse(result);
        if(playResponse == null){
            LOGGER.info("解析响应数据失败");
            return 0;
        }

        WxCuspayResp wxCuspayResp = InitWxCuspayRespData(playResponse);
        if(wxCuspayResp == null){
            return 0;
        }
        if(wxCuspayRespService.add(wxCuspayResp) == null){
            String info = "新增回調数据失败";
            LOGGER.error("--- {}", info);
            return 0;
        }
        //查找支付订单状态表
        Map<String, Object> reqParam = ImmutableMap.of("wxCuspaySn", playResponse.getMerchErpOrderSn(),"sendStatus", Contants.DECLARATION_SUCCESS_STATUS);
        WxCuspayStatus wxCuspayStatus = wxCuspayStatusService.findWxCuspayStatus(reqParam);
        if(wxCuspayStatus == null){
            String info = "查找支付订单状态表失败";
            LOGGER.error("--- {}", info);
            return 0;
        }
        WxPayDoc wxPayDoc = wxPayDocService.get(playResponse.getMerchErpOrderSn());
        if(wxPayDoc == null){
            String info = "查找支付单数据失败";
            LOGGER.error("--- {}", info);
            return 0;
        }

        WxOrder wxOrder = wxOrderService.get(wxPayDoc.getOrderSn());
        if(wxOrder == null){
            String info = "查找微信订单数据失败";
            LOGGER.error("--- {}", info);
            return 0;
        }

        if(CcnetContants.docStatus.item_12.getItem().equals(playResponse.getCusDeclStatus())){
            wxCuspayStatus.setSendStatus(Contants.SUCCESS_STATUS);
            wxCuspayStatus.setRespStatus(Contants.SUCCESS_STATUS);
            wxOrder.setPayNo(playResponse.getMerchOrderSn());
            wxOrder.setBuyerPayCheck(playResponse.getBuyerPayerCheck());
            wxOrder.setPlayOrderStatus(WmContants.wmStatuses.item_2.getItem());
            Map<String, Object> recordMap = ImmutableMap.of("orderSn", wxOrder.getOrderSn(), "paymentSuccTime", new Date(),"isPaymentSend", Contants.WX_SUCC);
            wxOrderGrabService.updateMallOrderProcessRecord(recordMap);
            Map<String, Object> wxMap = ImmutableMap.of("orderSn", wxOrder.getOrderSn(), "buyerPayCheck", playResponse.getBuyerPayerCheck());
            wxOrderGrabService.updateBuyerPayCheck(wxMap);

        }else if(CcnetContants.docStatus.item_13.getItem().equals(playResponse.getCusDeclStatus())){
            wxCuspayStatus.setSendStatus(Contants.FAIL_STATUS);
            wxCuspayStatus.setRespStatus(Contants.FAIL_STATUS);
            wxOrder.setPayNo(playResponse.getAllPayNo());
            wxOrder.setPlayOrderStatus(WmContants.wmStatuses.item_3.getItem());
            Map<String, Object> recordMap = ImmutableMap.of("orderSn", wxOrder.getOrderSn(), "paymentSuccTime", new Date(),"isPaymentSend", Contants.WX_FAIL);
            wxOrderGrabService.updateMallOrderProcessRecord(recordMap);

        }

        if(wxCuspayStatusService.update(wxCuspayStatus) == null){
            String info = "更新微信支付单状态失败";
            LOGGER.error("--- {}", info);
            return 0;
        }

        wxOrderService.update(wxOrder);
        LOGGER.info("海关处理回调成功");
        return 1;
    }

    /**
     * 解析响应数据
     * @param result
     * @return
     */
    private static PlayResponse analysisResponse(Result result) {
        Object dataResult = result.getData();
        if(dataResult == null){
            LOGGER.info("解析响应数据OrderResponse集合失败");
            return null;
        }
        Map<String, Object> dataMap = (Map<String, Object>) dataResult;
        PlayResponse playResponse = null;
        try {
            playResponse = (PlayResponse) MapBeanUtil.toObject(dataMap,PlayResponse.class);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return playResponse;

    }

    /**
     * 添加回调信息
     * @param playResponse
     * @return
     */
    private WxCuspayResp InitWxCuspayRespData(PlayResponse playResponse) {
        WxCuspayResp wxCuspayResp = new WxCuspayResp();
        String WxcuspayRespSn = SysContants.wx_cuspay_resp + SnowflakeUtil.getSnowNextId();;
        wxCuspayResp.setWxCuspayRespSn(WxcuspayRespSn);
        wxCuspayResp.setWxCuspaySn(playResponse.getMerchErpOrderSn());
        wxCuspayResp.setMerchSn(playResponse.getMerchSn());
        wxCuspayResp.setMerchName(playResponse.getMerchName());
        wxCuspayResp.setPlatSn(playResponse.getPlatSn());
        wxCuspayResp.setPlatName(playResponse.getPlatName());
        wxCuspayResp.setCusDeclStatus(playResponse.getCusDeclStatus());
        wxCuspayResp.setCode(playResponse.getCode());
        wxCuspayResp.setMsg(playResponse.getMsg());
        wxCuspayResp.setNote(playResponse.getBuyerPayerCheck());
        return  wxCuspayResp;
    }
}
