package com.emato.ccnet.wx.biz.req.play.wx;

import com.emato.ccnet.wx.biz.CebReqSendDate;
import com.emato.ccnet.wx.config.Cus;
import com.emato.ccnet.wx.contants.Contants;
import com.emato.ccnet.wx.dao.entity.ccnet.WxCuspayStatus;
import com.emato.ccnet.wx.dao.entity.ccnet.WxPayDoc;
import com.emato.ccnet.wx.dto.ccnetdto.WxPayDto;
import com.emato.ccnet.wx.service.WxCuspayStatusService;
import com.emato.ccnet.wx.service.WxPayDocService;
import com.emato.ccnet.wx.support.msg.resp.ResponseMessage;
import com.emato.ccnet.wx.util.BeanUtil;
import com.emato.ccnet.wx.util.OkHttpUtils;
import com.emato.ccnet.wx.util.jackson.JacksonUtil;
import com.google.common.base.Joiner;
import com.google.common.collect.ImmutableMap;
import com.google.common.collect.Lists;
import okhttp3.Request;
import okhttp3.RequestBody;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.util.*;

@Component
public class WxplaySendDateRequet implements CebReqSendDate {
    private static final Logger LOGGER = LoggerFactory.getLogger(WxplaySendDateRequet.class);

    public final String CUS_PLAY_EXCLUDE = "wxCuspaySn,orderSn,remark,createrSn,createTime,moderSn,modTime,tstm,exField,exField2,exField3,exField4,exField5";

    @Autowired
    private WxPayDocService wxPayDocService;

    @Autowired
    private WxplaySendDateResponse wxplaySendDateResponse;

    @Autowired
    private WxplaySendDateAfter wxplaySendDateAfter;

    @Autowired
    private WxCuspayStatusService wxCuspayStatusService;

    @Autowired
    private Cus cus;

    /**
     * 发送请求
     * @return
     */
    @Override
    public ResponseMessage sendDate() {
        //查找当前wm_ccnet中的所有支付订单
        List<WxPayDoc> wxPayDocs = wxPayDocService.findAllWxPayDoc();
        if(wxPayDocs == null){
            String info = "查找当前wm_ccnet中的所有支付订单失败";
            LOGGER.error("--- {}", info);
            return ResponseMessage.failed(info);
        }

        //发送给第三方的请求回调
        ResponseMessage wxPaySumbit = buildRequest(wxPayDocs);
        if(!ResponseMessage.isSuccess(wxPaySumbit)){
            String info = "查找当前ccnet中的所有电子订单失败";
            LOGGER.error("--- {}", info);
            return ResponseMessage.failed(info);
        }
        return ResponseMessage.success();
    }

    /**
     * 发送第三方请求和回调
     * @param wxPayDocs
     * @return
     */
    private ResponseMessage buildRequest(List<WxPayDoc> wxPayDocs) {

        List<String> sucList = new ArrayList<>();
        List<String> failedList = new ArrayList<>();
        for (WxPayDoc wxPayDoc : wxPayDocs){

            //更新支付单数据为申报中
            ResponseMessage updateWxCuspayStatus = updateWxCuspayStatus(wxPayDoc.getWxCuspaySn());
            if(!ResponseMessage.isSuccess(updateWxCuspayStatus)){
                failedList.add(wxPayDoc.getSubOrderNo());
                String info = "更新支付单数据为申报中失败" + updateWxCuspayStatus.getMsg();
                LOGGER.error("--- {}", info);
                continue;
            }

            //数据组装
            Map<String,String> sPara = null;
            try {
                sPara = buildRequestDate(wxPayDoc);
                if(sPara == null){
                    failedList.add(wxPayDoc.getSubOrderNo());
                    String info = "数据组装失败";
                    LOGGER.error("--- {}", info);
                    continue;
                }
            } catch (Exception e) {
                failedList.add(wxPayDoc.getSubOrderNo());
                String info = "数据组装失败";
                LOGGER.error("--- {}", info);
                continue;
            }

            //发送CSP系统请求并获取响应信息
            String response = requestSubmit(sPara);
            LOGGER.info("WxPlay的响应报文：" + response);
            if(response == null){
                failedList.add(wxPayDoc.getSubOrderNo());
                String info = "发送请求并获取响应信息失败";
                LOGGER.error("--- {}", info);
                continue;
            }


            LOGGER.info("response date:"+response);

            try {
                //解析响应报文并处理本地业务逻辑
                ResponseMessage responseDis = wxplaySendDateResponse.addResponse(response,wxPayDoc.getWxCuspaySn());
                if(!ResponseMessage.isSuccess(responseDis)){
                    failedList.add(wxPayDoc.getOrderSn());
                    String info = "解析响应报文并处理本地业务逻辑失败";
                    LOGGER.error("--- {}", info);
                    continue;
                }
            } catch (Exception e) {
                failedList.add(wxPayDoc.getSubOrderNo());
                String info = "解析响应报文并处理本地业务逻辑失败";
                LOGGER.error("--- {}", info);
                continue;
            }

            sucList.add(wxPayDoc.getSubOrderNo());
        }

        LOGGER.info("--- 【WMPlay】【成功】订单数:{}, 订单: {}", sucList.size(), Joiner.on(",").skipNulls().join(sucList.toArray()));
        LOGGER.info("--- 【WMPlay】【失败】订单数:{}, 订单: {}", failedList.size(), Joiner.on(",").skipNulls().join(failedList.toArray()));

        wxplaySendDateAfter.afterSend(sucList,failedList);

        return ResponseMessage.success();
    }
    /**
     * 发送请求并获取响应信息
     * @param sPara
     * @return
     */

    private String requestSubmit(Map<String, String> sPara) {
        //构造RequestBody
        RequestBody requestBody = OkHttpUtils.buildRequestBody(sPara);

        //构建Request
        String url = ((Map<String, Object>) cus.getPlay().get("wx")).get("play-url").toString();
        Request request = OkHttpUtils.buildRequest(url,requestBody);

        // 同步访问,返回结果字符串
        String response = null;
        try {
            response = OkHttpUtils.post(request);
        } catch (IOException e) {
            String info = "同步访问,返回结果字符串错误";
            LOGGER.error("--- {}", info);
            return null;
        }

        return response;
    }

    /**
     * 更新电子订单状态表
     * @param wxCuspaySn
     * @return
     */
    private ResponseMessage updateWxCuspayStatus(String wxCuspaySn) {
        //通过编号查找电子订单表
        Map<String, Object> reqParam = ImmutableMap.of("wxCuspaySn", wxCuspaySn,"sendStatus", Contants.INIT_STATUS);
        WxCuspayStatus wxCuspayStatus = wxCuspayStatusService.findWxCuspayStatus(reqParam);
        if(wxCuspayStatus == null){
            return ResponseMessage.failed("该电子订单不存在");
        }

        //更新请求报文状态:请求申报中
        wxCuspayStatus.setModerSn("");
        wxCuspayStatus.setModTime(new Date());
        wxCuspayStatus.setSendStatus(Contants.DECLARATION_STATUS);
        WxCuspayStatus cuspayStatus = wxCuspayStatusService.update(wxCuspayStatus);
        if(cuspayStatus == null){
            return ResponseMessage.failed("该电子订单更新失败");
        }
        return ResponseMessage.success();
    }

    /**
     * 构造请求参数
     * @param wxPayDoc
     * @return
     */
    private Map<String, String> buildRequestDate(WxPayDoc wxPayDoc) {
        // 发送电子订单的集合
        Map<String, String> sPara = new HashMap<>();
        List list = Lists.newArrayList();
        WxPayDto wxPayDto = new WxPayDto();
        BeanUtil.copy(wxPayDoc, wxPayDto);
        wxPayDto.setMerchErpOrderSn(wxPayDoc.getWxCuspaySn());
        list.add(wxPayDto);
        String requestData = JacksonUtil.toJson(list);
        sPara.put("data", requestData);
        LOGGER.info("request Data :" + requestData);
        return sPara;

    }


}
