package com.emato.ccnet.wx.biz.req.play.wx;

import com.emato.ccnet.wx.biz.CebRespDateRespons;
import com.emato.ccnet.wx.contants.Contants;
import com.emato.ccnet.wx.contants.SysContants;
import com.emato.ccnet.wx.dao.entity.ccnet.WxCuspayResp;
import com.emato.ccnet.wx.dao.entity.ccnet.WxCuspayStatus;
import com.emato.ccnet.wx.dao.entity.ccnet.WxPayDoc;
import com.emato.ccnet.wx.manager.result.Result;
import com.emato.ccnet.wx.manager.snow.SnowflakeUtil;
import com.emato.ccnet.wx.service.WxCuspayRespService;
import com.emato.ccnet.wx.service.WxCuspayStatusService;
import com.emato.ccnet.wx.service.WxOrderGrabService;
import com.emato.ccnet.wx.service.WxPayDocService;
import com.emato.ccnet.wx.support.msg.resp.ResponseMessage;
import com.google.common.collect.ImmutableMap;
import com.google.gson.Gson;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.Map;

@Component
public class WxplaySendDateResponse implements CebRespDateRespons {
    private static final Logger LOGGER = LoggerFactory.getLogger(WxplaySendDateResponse.class);

    @Autowired
    private WxCuspayStatusService wxCuspayStatusService;

    @Autowired
    private WxCuspayRespService wxCuspayRespService;

    @Autowired
    private WxOrderGrabService wxOrderGrabService;

    @Autowired
    private WxPayDocService wxPayDocService;

    @Override
    public ResponseMessage addResponse(String response, String wxCuspaySn) {

        Gson gson = new Gson();
        Result result = gson.fromJson(response, Result.class);
        if(result == null){
            LOGGER.info("解析响应数据Result失败");
            return null;
        }
        //查找电子订单状态表
        Map<String, Object> reqParam = ImmutableMap.of("wxCuspaySn", wxCuspaySn,"sendStatus", Contants.DECLARATION_STATUS);
        WxCuspayStatus wxCuspayStatus = wxCuspayStatusService.findWxCuspayStatus(reqParam);
        WxCuspayResp wxCuspayResp = null;


        //新增回調数据
        wxCuspayResp = InitWxCuspayRespData(wxCuspayStatus);
        wxCuspayResp.setReturnCode(result.getCode());
        wxCuspayResp.setReturnMsg(result.getMsg());

        if(wxCuspayRespService.add(wxCuspayResp) == null){
            String info = "新增回調数据失败";
            LOGGER.error("--- {}", info);
            return ResponseMessage.failed(info);
        }
        WxPayDoc wxPayDoc = wxPayDocService.get(wxCuspaySn);
        if (!"0".equals(result.getCode())){
            wxCuspayStatus.setSendStatus(Contants.FAIL_STATUS);
            if(wxCuspayStatusService.update(wxCuspayStatus) == null){
                String info = "更新微信支付单状态失败";
                LOGGER.error("--- {}", info);
                return ResponseMessage.failed(info);
            }
            Map<String, Object> recordMap = ImmutableMap.of("orderSn", wxPayDoc.getSubOrderNo(), "paymentStartTime", new Date(),"isPaymentSend", Contants.WX_FAIL);
            wxOrderGrabService.updateMallOrderProcessRecord(recordMap);
            return ResponseMessage.failed();
        }else{
            wxCuspayStatus.setSendStatus(Contants.DECLARATION_SUCCESS_STATUS);
            wxCuspayStatus.setRespStatus(Contants.SENDING_STATUS);
            if(wxCuspayStatusService.update(wxCuspayStatus) == null){
                String info = "更新微信支付单状态失败";
                LOGGER.error("--- {}", info);
                return ResponseMessage.failed(info);
            }
            Map<String, Object> recordMap = ImmutableMap.of("orderSn", wxPayDoc.getSubOrderNo(), "paymentStartTime", new Date(),"isPaymentSend", Contants.WX_UNDERWAY);
            wxOrderGrabService.updateMallOrderProcessRecord(recordMap);
            return ResponseMessage.success();
        }



    }

    private WxCuspayResp InitWxCuspayRespData(WxCuspayStatus wxCuspayStatus) {
        WxCuspayResp wxCuspayResp = new WxCuspayResp();
        String WxcuspayRespSn = SysContants.wx_cuspay_resp + SnowflakeUtil.getSnowNextId();;
        wxCuspayResp.setWxCuspayRespSn(WxcuspayRespSn);
        wxCuspayResp.setWxCuspaySn(wxCuspayStatus.getWxPaySn());
        wxCuspayResp.setMerchSn(wxCuspayStatus.getMerchSn());
        wxCuspayResp.setMerchName(wxCuspayStatus.getMerchName());
        wxCuspayResp.setPlatSn(wxCuspayStatus.getPlatSn());
        wxCuspayResp.setPlatName(wxCuspayStatus.getPlatName());
        return  wxCuspayResp;
    }

}
