package com.emato.ccnet.wx.biz.req.waybill.oms;

import com.emato.ccnet.wx.biz.CebReqBiz;
import com.emato.ccnet.wx.contants.CcnetContants;
import com.emato.ccnet.wx.contants.Contants;
import com.emato.ccnet.wx.dao.entity.ccnet.WxOrder;
import com.emato.ccnet.wx.service.WxOrderService;
import com.emato.ccnet.wx.support.msg.resp.ResponseMessage;
import com.google.common.collect.ImmutableMap;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Component
public class Ceb511OmsWayBiz implements CebReqBiz {
    private static final Logger LOGGER = LoggerFactory.getLogger(Ceb511OmsWayBiz.class);

    @Autowired
    private Ceb511OmsWayGetDate ceb511OmsWayGetDate;

    @Autowired
    private Ceb511OmsWayAddDate ceb511OmsWayAddDate;

    @Autowired
    private Ceb511SendDateRequest ceb511SendDateRequest;

    @Autowired
    private WxOrderService wxOrderService;

    @Value("${db.ccnet-send-wm.page-size}")
    private String limit = "20";

    /**
     * 业务入口
     * @return
     */
    @Override
    public ResponseMessage biz() {
        Map<String,Object> reqParam = ImmutableMap.of("number", Integer.valueOf(limit),"wxOrderDetail", CcnetContants.isscan.item_1.getItem(),
                "wayOrderStatus", CcnetContants.sendStatus.item_0.getItem(),"playOrderStatus",CcnetContants.sendStatus.item_2.getItem(),"buyerPayCheck",CcnetContants.isscan.item_1.getItem());
        //查询海关运单数据
        List list = ceb511OmsWayGetDate.getData(reqParam);
        if (list.isEmpty()) {
            String info = "【CDB511】没有订单数据, 提前结束";
            LOGGER.error("--- {}", info);
            afterFailed(list);
            return ResponseMessage.failed(info);
        }

        //添加海关运单数据
        ResponseMessage responseMessage = ceb511OmsWayAddDate.addDate(list);
        if(!ResponseMessage.isSuccess(responseMessage)){
            String info = "生成海关运单初始化失败" + responseMessage.getMsg();
            LOGGER.error("--- {}", info);
            afterFailed(list);
            return ResponseMessage.failed(info);
        }

        //发送数据请求并响应
        ResponseMessage ceb511ResponseMessage = ceb511SendDateRequest.sendDate();
        if(!ResponseMessage.isSuccess(ceb511ResponseMessage)){
            String info = "发送请求响应失败" + ceb511ResponseMessage.getMsg();
            LOGGER.error("--- {}", info);
            afterFailed(list);
            return ResponseMessage.failed(info);
        }

        return ResponseMessage.success();
    }

    private void afterFailed(List list) {
        List<String> failedList = new ArrayList<>();
        List<Map<String, Object>> wmFaileds = new ArrayList<>();

        list.stream().forEach(x->{
            failedList.add(((WxOrder)x).getOrderSn());
        });
        if (failedList.size() > 0) {
            failedList.stream().forEach(x->{
                Map<String, Object> wmMap = ImmutableMap.of("orderSn", x,  "wayOrderStatus", Contants.FAIL_STATUS);
                wmFaileds.add(wmMap);
            });
            if (wmFaileds.size() > 0) {
                wxOrderService.updateWxOrderStatusBatch(wmFaileds);
            }
        }
    }

}
