package com.emato.ccnet.wx.biz.req.waybill.oms;

import com.emato.ccnet.wx.biz.CebReqSendDateAfter;
import com.emato.ccnet.wx.contants.Contants;
import com.emato.ccnet.wx.service.CusWaybillStatusService;
import com.emato.ccnet.wx.service.WxOrderService;
import com.google.common.collect.ImmutableMap;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 海关电子订单 运单 报文发送后
 * @author Scott Chen
 * @version 1.0
 * 2017-11-08 14:14
 */
@Component
public class Ceb511SendDateAfter implements CebReqSendDateAfter {
    private static final Logger LOGGER = LoggerFactory.getLogger(Ceb511SendDateAfter.class);

    @Autowired
    private CusWaybillStatusService cusWaybillStatusService;

    @Autowired
    private WxOrderService wxOrderService;

    /**
     * 更新状态
     * @param sucList
     * @param failedList
     * @return
     */
    @Override
    public boolean afterSend(List<String> sucList, List<String> failedList) {
        List<Map<String, Object>> wxPlaySucs = new ArrayList<>();
        List<Map<String, Object>> wxPlayFaileds = new ArrayList<>();
        List<Map<String, Object>> wxOrderFaileds = new ArrayList<>();

        //全部处理完后
        if (sucList.size() > 0) {
            LOGGER.info("--- 【CDB511】更新【成功】报文订单记录......");
            sucList.stream().forEach(x->{
                Map<String, Object> wxPlayMap = ImmutableMap.of("merchOrderSn", x, "sendStatus", Contants.DECLARATION_SUCCESS_STATUS);
                wxPlaySucs.add(wxPlayMap);
            });
            if(wxPlaySucs.size() > 0){
                try {
                    if(wxPlaySucs.size() > 0) {
                        cusWaybillStatusService.updateCusWaybillStatusBatch(wxPlaySucs);
                    }
                } catch (Exception e) {
                    String info = "【CDB511中的wx推单】更新【成功】报文订单记录失败";
                    LOGGER.error("--- {}", info);
                    return false;
                }
            }
        }

        if (failedList.size() > 0) {
            LOGGER.info("--- 【CDB511】更新【失败】报文订单记录......");
            failedList.stream().forEach(x->{
                Map<String, Object> wxPlayMap = ImmutableMap.of("merchOrderSn", x,  "sendStatus", Contants.FAIL_STATUS);
                Map<String, Object> map = ImmutableMap.of("orderSn", x, "wayOrderStatus", Contants.FAIL_STATUS);
                wxPlayFaileds.add(wxPlayMap);
                wxOrderFaileds.add(map);
            });

            try {
                if(wxOrderFaileds.size() > 0) {
                    wxOrderService.updateWxOrderStatusBatch(wxOrderFaileds);
                }
                if(wxPlayFaileds.size() > 0) {
                    cusWaybillStatusService.updateCusWaybillStatusBatch(wxPlayFaileds);
                }
            } catch (Exception e) {
                String info = "【CDB511中的wx推单】更新【失败】报文订单记录失败";
                LOGGER.error("--- {}", info);
                return false;
            }
        }

        return true;
    }
}
