package com.emato.ccnet.wx.biz.req.waybill.oms;

import com.emato.ccnet.wx.biz.CebRespDateRespons;
import com.emato.ccnet.wx.contants.Contants;
import com.emato.ccnet.wx.dao.entity.ccnet.CusWaybillStatus;
import com.emato.ccnet.wx.service.CusWaybillRespService;
import com.emato.ccnet.wx.service.CusWaybillStatusService;
import com.emato.ccnet.wx.support.msg.resp.ResponseMessage;
import com.google.common.collect.ImmutableMap;
import com.google.gson.Gson;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.Map;

@Component
public class Ceb511SendDateResponse implements CebRespDateRespons {
    private static final Logger LOGGER = LoggerFactory.getLogger(Ceb511SendDateResponse.class);
    @Autowired
    private CusWaybillStatusService cusWaybillStatusService;

    @Autowired
    private CusWaybillRespService cusWaybillRespService;
    /**
     * 对响应进行处理
     * @param response
     * @param cusWaybillSn
     * @return
     */
    @Override
    public ResponseMessage addResponse(String response, String cusWaybillSn) {
        //解析响应数据
        Gson gson = new Gson();
        ResponseMessage result = gson.fromJson(response, ResponseMessage.class);
        if(result == null){
            String info = "解析响应数据Result失败";
            LOGGER.error("--- {}", info);
            return ResponseMessage.failed(info);
        }

        //新增回调数据
        int cusWaybillResp = cusWaybillRespService.insertWaybillResp(result,cusWaybillSn);
        if (cusWaybillResp != 1){
            String info = "回调数据添加失败";
            LOGGER.error("--- {}", info);
            return ResponseMessage.failed(info);
        }

        //查找运单状态表
        Map<String, Object> reqParam = ImmutableMap.of("cusWaybillSn", cusWaybillSn,"sendStatus", Contants.SENDING_STATUS);
        CusWaybillStatus cusWaybillStatus = cusWaybillStatusService.findCusWaybillStatus(reqParam);
        if(cusWaybillStatus == null){
            String info = "查找运单状态表数据失败";
            LOGGER.error("--- {}", info);
            return ResponseMessage.failed(info);
        }

        //判断结果
        if(Contants.OMS_SUCCESS_CODE.equals(result.getCode())){
            cusWaybillStatus.setRespStatus(Contants.SUCCESS_STATUS);
            cusWaybillStatus.setModerSn(Contants.CREATER_NUMBER);
            cusWaybillStatus.setModTime(new Date());
            cusWaybillStatusService.updateCusWaybillStatus(cusWaybillStatus);

            return ResponseMessage.success();
        }else{
            cusWaybillStatus.setRespStatus(Contants.FAIL_STATUS);
            cusWaybillStatus.setModerSn(Contants.CREATER_NUMBER);
            cusWaybillStatus.setModTime(new Date());
            cusWaybillStatusService.updateCusWaybillStatus(cusWaybillStatus);

            String info = result.getMsg();
            LOGGER.error("--- {}", info);
            return ResponseMessage.failed(info);
        }

    }


}
