package com.emato.ccnet.wx.config.db.middle;

import org.apache.ibatis.session.SqlSessionFactory;
import org.mybatis.spring.SqlSessionFactoryBean;
import org.mybatis.spring.SqlSessionTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.jdbc.DataSourceProperties;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.env.Environment;
import org.springframework.core.io.support.PathMatchingResourcePatternResolver;
import org.springframework.jdbc.datasource.DataSourceTransactionManager;

import javax.sql.DataSource;

/**
 * 多数据源配置
 *
 * 数据源配置
 *
 * 指定mybatis-config.xml, 和要扫描的别名包
 *
 * @author Scott Chen
 * @date 2017/4/22
 */
@Configuration
public class MiddleDataSourceConfiguration {

    @Autowired
    private Environment env;

    @Bean
    @ConfigurationProperties(prefix = "spring.datasource.middle")
    public DataSourceProperties middleDataSourceProperties() {
        return new DataSourceProperties();
    }

    @Bean
    public DataSource middleDataSource() {
        DataSource dataSource = middleDataSourceProperties().initializeDataSourceBuilder().build();
        return dataSource;
    }

    @Bean
    public SqlSessionFactory middleSqlSessionFactory(DataSource middleDataSource) throws Exception  {
        SqlSessionFactoryBean sessionFactoryBean = new SqlSessionFactoryBean();
        sessionFactoryBean.setDataSource(middleDataSource);

        //配置mybatis-config.xml
        sessionFactoryBean.setConfigLocation(new PathMatchingResourcePatternResolver().getResource(env.getProperty("mybatis.middle.config-location")));
        //配置mapper.xml
        sessionFactoryBean.setMapperLocations(new PathMatchingResourcePatternResolver().getResources(env.getProperty("mybatis.middle.mapper-locations")));
        //配置扫描type aliases的包路径
        sessionFactoryBean.setTypeAliasesPackage(env.getProperty("mybatis.middle.type-aliases-package"));
        return sessionFactoryBean.getObject();
    }

    @Bean(name = "middleTransactionManager")
    public DataSourceTransactionManager middleTransactionManager( DataSource middleDataSource) {
        return new DataSourceTransactionManager(middleDataSource);
    }

    @Bean
    public SqlSessionTemplate middleSqlSessionTemplate(SqlSessionFactory middleSqlSessionFactory) {
        return new SqlSessionTemplate(middleSqlSessionFactory);
    }

}
