package com.emato.ccnet.wx.config.db.three;

import org.apache.ibatis.session.SqlSessionFactory;
import org.mybatis.spring.SqlSessionFactoryBean;
import org.mybatis.spring.SqlSessionTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.jdbc.DataSourceProperties;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.env.Environment;
import org.springframework.core.io.support.PathMatchingResourcePatternResolver;
import org.springframework.jdbc.datasource.DataSourceTransactionManager;

import javax.sql.DataSource;

/**
 * 多数据源配置
 *
 * 数据源配置
 *
 * 指定mybatis-config.xml, 和要扫描的别名包
 *
 * @author Scott Chen
 * @date 2017/4/22
 */
@Configuration
public class ThreeDataSourceConfiguration {

    @Autowired
    private Environment env;

    @Bean
    @ConfigurationProperties(prefix = "spring.datasource.three")
    public DataSourceProperties threeDataSourceProperties() {
        return new DataSourceProperties();
    }

    @Bean
    public DataSource threeDataSource() {
        DataSource dataSource = threeDataSourceProperties().initializeDataSourceBuilder().build();
        return dataSource;
    }

    @Bean
    public SqlSessionFactory threeSqlSessionFactory(DataSource threeDataSource) throws Exception  {
        SqlSessionFactoryBean sessionFactoryBean = new SqlSessionFactoryBean();
        sessionFactoryBean.setDataSource(threeDataSource);

        //配置mybatis-config.xml
        sessionFactoryBean.setConfigLocation(new PathMatchingResourcePatternResolver().getResource(env.getProperty("mybatis.three.config-location")));
        //配置mapper.xml
        sessionFactoryBean.setMapperLocations(new PathMatchingResourcePatternResolver().getResources(env.getProperty("mybatis.three.mapper-locations")));
        //配置扫描type aliases的包路径
        sessionFactoryBean.setTypeAliasesPackage(env.getProperty("mybatis.three.type-aliases-package"));
        return sessionFactoryBean.getObject();
    }

    @Bean(name = "threeTransactionManager")
    public DataSourceTransactionManager threeTransactionManager( DataSource threeDataSource) {
        return new DataSourceTransactionManager(threeDataSource);
    }

    @Bean
    public SqlSessionTemplate threeSqlSessionTemplate(SqlSessionFactory threeSqlSessionFactory) {
        return new SqlSessionTemplate(threeSqlSessionFactory);
    }

}
