package com.emato.ccnet.wx.dto.ccnetdto;

import com.emato.ccnet.wx.dao.entity.ccnet.CusWaybill;
import com.fasterxml.jackson.annotation.JsonFormat;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

/**
 * 保税备货
 * @author zengjunlin
 * @version 1.0
 * 2017-10-26 11:03
 */
public class BondedStockMerchantOrder implements Serializable {
    private static final long serialVersionUID = 3577514804854115706L;
    //商户编号
    private String merchId;
    //商户名称
    private String merchName;
    // 第三方商户代码
    private String thirdPartyMerchCode;
    //第三方商户名称
    private String thirdPartyMerchName;
    //商户订单编号
    private String merchOrderId;
    //订购人证件类型
    private String buyerIdType;
    //订购人证件号码
    private String buyerIdCode;
    //订购人姓名
    private String buyerName;
    //订购人电话
    private String buyerTel;
    //支付人证件类型
    private String payerIdType;
    //支付人证件号码
    private String payerIdCode;
    //支付人姓名
    private String payerName;
    //支付人电话
    private String payerMob;
    //收件人证件类型
    private String consigneeIdType;
    //收件人证件号码
    private String consigneeIdCode;
    //收件人姓名
    private String consigneeName;
    //收件人手机
    private String consigneeMob;
    //收件人电话
    private String consigneeTel;
    //收件人国家代码
    private String consigneeCountryCode;
    //收件人省区划代码
    private String consigneeProvinceCode;
    //收件人省名
    private String consigneeProvince;
    //收件人市区划代码
    private String consigneeCityCode;
    //收件人市名
    private String consigneeCity;
    //收件人区县区划代码
    private String consigneeDistrictCode;
    //收件人区县
    private String consigneeDistrict;
    //收件人地址
    private String consigneeAddress;
    //收件人邮政编码
    private String consigneeZipCode;
    //支付企业海关登记号
    private String payEntCusCode;
    //支付单号
    private String payNo;
    //实际支付金额
    private BigDecimal acturalPaid;
    //支付时间
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date payTime;
    //快递类型
    private String exprType;
    //快递公司编号
    private String exprCompId;
    //订购人下单时间
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date buyerBillTime;
    //申报用物流费
    private BigDecimal declExprFee;
    //申报用行邮税
    private BigDecimal declPostTax;
    //商户字段
    private String extraTag;
    //备注
    private String remark;
    //平台编码
    private String platSn;
    // 门店编号
    private String shopId;
    //海关支付推送成功
    private String cusPaymentSucceeded;
    // 海关订单推送成功
    private String cusOrderSucceeded;
    // 支付成功
    private String paySucceeded;
    // 订购人支付人核验
    private String buyerPayerCheck;
    // 快递协议类型
    private String exprAgreementType;
    // 运输方式
    private String transMode;
    //商品详情
    private List<BondedStockMerchantOrderItem> item;

    public BondedStockMerchantOrder(){}

    public BondedStockMerchantOrder(CusWaybill cusWaybill){
        this.merchId = cusWaybill.getMerchSn();
        this.merchOrderId = cusWaybill.getMerchOrderId();
        this.thirdPartyMerchCode = cusWaybill.getThirdPartyMerchCode();
        this.thirdPartyMerchName = cusWaybill.getThirdPartyMerchName();
        this.buyerIdType = cusWaybill.getBuyerIdType();
        this.buyerIdCode = cusWaybill.getBuyerIdCode() == null?cusWaybill.getConsigneeIdCode():cusWaybill.getBuyerIdCode();
        this.buyerName = cusWaybill.getBuyerName();
        this.buyerTel = cusWaybill.getBuyerTel();
        this.consigneeIdType = cusWaybill.getConsigneeIdType();
        this.consigneeIdCode = cusWaybill.getConsigneeIdCode();
        this.consigneeName = cusWaybill.getConsigneeName();
        this.consigneeMob = cusWaybill.getConsigneeMob();
        this.consigneeTel = cusWaybill.getConsigneeTel();
        this.consigneeCountryCode = cusWaybill.getConsigneeCountryCode();
        this.consigneeProvinceCode = cusWaybill.getConsigneeProvinceCode();
        this.consigneeProvince = cusWaybill.getConsigneeProvince();
        this.consigneeCityCode = cusWaybill.getConsigneeCityCode();
        this.consigneeCity = cusWaybill.getConsigneeCity();
        this.consigneeDistrictCode = cusWaybill.getConsigneeDistrictCode();
        this.consigneeDistrict = cusWaybill.getConsigneeDistrict();
        this.consigneeAddress = cusWaybill.getConsigneeAddress();
        this.consigneeZipCode = cusWaybill.getConsigneeZipCode();
        this.payerIdType = cusWaybill.getPayerIdType();
        this.payerIdCode = cusWaybill.getPayerIdCode();
        this.payerName = cusWaybill.getPayerName();
        this.payerMob = cusWaybill.getPayerMob();
        this.payEntCusCode = cusWaybill.getPayEntCusCode();
        this.payNo = cusWaybill.getPayNo();
        this.acturalPaid = cusWaybill.getActuralPaid();
        this.payTime = cusWaybill.getPayTime();
        this.buyerBillTime = cusWaybill.getBuyerBillTime()==null?new Date():cusWaybill.getBuyerBillTime();
        this.extraTag = cusWaybill.getCusWaybillSn();
        this.exprType = cusWaybill.getExprType();
        this.exprCompId = cusWaybill.getExprCompId();
        this.declExprFee = BigDecimal.valueOf(0);
        this.declPostTax = BigDecimal.valueOf(0);
        this.shopId = cusWaybill.getShopId();
        this.cusPaymentSucceeded = "0";
        this.cusOrderSucceeded = "0";
        this.paySucceeded = "0";
        System.out.println(cusWaybill.getExField());
        this.buyerPayerCheck = cusWaybill.getBuyerPayCheck();
        this.exprAgreementType = cusWaybill.getExprAgreementType();
        this.transMode = "20";
    }

    public String getMerchId() {
        return merchId;
    }

    public void setMerchId(String merchId) {
        this.merchId = merchId;
    }

    public String getMerchOrderId() {
        return merchOrderId;
    }

    public void setMerchOrderId(String merchOrderId) {
        this.merchOrderId = merchOrderId;
    }

    public String getBuyerIdType() {
        return buyerIdType;
    }

    public void setBuyerIdType(String buyerIdType) {
        this.buyerIdType = buyerIdType;
    }

    public String getBuyerIdCode() {
        return buyerIdCode;
    }

    public void setBuyerIdCode(String buyerIdCode) {
        this.buyerIdCode = buyerIdCode;
    }

    public String getBuyerName() {
        return buyerName;
    }

    public void setBuyerName(String buyerName) {
        this.buyerName = buyerName;
    }

    public String getBuyerTel() {
        return buyerTel;
    }

    public void setBuyerTel(String buyerTel) {
        this.buyerTel = buyerTel;
    }

    public String getPayerIdType() {
        return payerIdType;
    }

    public void setPayerIdType(String payerIdType) {
        this.payerIdType = payerIdType;
    }

    public String getPayerIdCode() {
        return payerIdCode;
    }

    public void setPayerIdCode(String payerIdCode) {
        this.payerIdCode = payerIdCode;
    }

    public String getPayerName() {
        return payerName;
    }

    public void setPayerName(String payerName) {
        this.payerName = payerName;
    }

    public String getPayerMob() {
        return payerMob;
    }

    public void setPayerMob(String payerMob) {
        this.payerMob = payerMob;
    }

    public String getConsigneeIdType() {
        return consigneeIdType;
    }

    public void setConsigneeIdType(String consigneeIdType) {
        this.consigneeIdType = consigneeIdType;
    }

    public String getConsigneeIdCode() {
        return consigneeIdCode;
    }

    public void setConsigneeIdCode(String consigneeIdCode) {
        this.consigneeIdCode = consigneeIdCode;
    }

    public String getConsigneeName() {
        return consigneeName;
    }

    public void setConsigneeName(String consigneeName) {
        this.consigneeName = consigneeName;
    }

    public String getConsigneeMob() {
        return consigneeMob;
    }

    public void setConsigneeMob(String consigneeMob) {
        this.consigneeMob = consigneeMob;
    }

    public String getConsigneeTel() {
        return consigneeTel;
    }

    public void setConsigneeTel(String consigneeTel) {
        this.consigneeTel = consigneeTel;
    }

    public String getConsigneeCountryCode() {
        return consigneeCountryCode;
    }

    public void setConsigneeCountryCode(String consigneeCountryCode) {
        this.consigneeCountryCode = consigneeCountryCode;
    }

    public String getConsigneeProvinceCode() {
        return consigneeProvinceCode;
    }

    public void setConsigneeProvinceCode(String consigneeProvinceCode) {
        this.consigneeProvinceCode = consigneeProvinceCode;
    }

    public String getConsigneeProvince() {
        return consigneeProvince;
    }

    public void setConsigneeProvince(String consigneeProvince) {
        this.consigneeProvince = consigneeProvince;
    }

    public String getConsigneeCityCode() {
        return consigneeCityCode;
    }

    public void setConsigneeCityCode(String consigneeCityCode) {
        this.consigneeCityCode = consigneeCityCode;
    }

    public String getConsigneeCity() {
        return consigneeCity;
    }

    public void setConsigneeCity(String consigneeCity) {
        this.consigneeCity = consigneeCity;
    }

    public String getConsigneeDistrictCode() {
        return consigneeDistrictCode;
    }

    public void setConsigneeDistrictCode(String consigneeDistrictCode) {
        this.consigneeDistrictCode = consigneeDistrictCode;
    }

    public String getConsigneeDistrict() {
        return consigneeDistrict;
    }

    public void setConsigneeDistrict(String consigneeDistrict) {
        this.consigneeDistrict = consigneeDistrict;
    }

    public String getConsigneeAddress() {
        return consigneeAddress;
    }

    public void setConsigneeAddress(String consigneeAddress) {
        this.consigneeAddress = consigneeAddress;
    }

    public String getConsigneeZipCode() {
        return consigneeZipCode;
    }

    public void setConsigneeZipCode(String consigneeZipCode) {
        this.consigneeZipCode = consigneeZipCode;
    }

    public String getPayEntCusCode() {
        return payEntCusCode;
    }

    public void setPayEntCusCode(String payEntCusCode) {
        this.payEntCusCode = payEntCusCode;
    }

    public String getPayNo() {
        return payNo;
    }

    public void setPayNo(String payNo) {
        this.payNo = payNo;
    }

    public BigDecimal getActuralPaid() {
        return acturalPaid;
    }

    public void setActuralPaid(BigDecimal acturalPaid) {
        this.acturalPaid = acturalPaid;
    }

    public Date getPayTime() {
        return payTime;
    }

    public void setPayTime(Date payTime) {
        this.payTime = payTime;
    }

    public String getExprType() {
        return exprType;
    }

    public void setExprType(String exprType) {
        this.exprType = exprType;
    }

    public String getExprCompId() {
        return exprCompId;
    }

    public void setExprCompId(String exprCompId) {
        this.exprCompId = exprCompId;
    }

    public Date getBuyerBillTime() {
        return buyerBillTime;
    }

    public void setBuyerBillTime(Date buyerBillTime) {
        this.buyerBillTime = buyerBillTime;
    }

    public BigDecimal getDeclExprFee() {
        return declExprFee;
    }

    public void setDeclExprFee(BigDecimal declExprFee) {
        this.declExprFee = declExprFee;
    }

    public BigDecimal getDeclPostTax() {
        return declPostTax;
    }

    public void setDeclPostTax(BigDecimal declPostTax) {
        this.declPostTax = declPostTax;
    }

    public String getExtraTag() {
        return extraTag;
    }

    public void setExtraTag(String extraTag) {
        this.extraTag = extraTag;
    }

    public String getRemark() {
        return remark;
    }

    public void setRemark(String remark) {
        this.remark = remark;
    }

    public List<BondedStockMerchantOrderItem> getItem() {
        return item;
    }

    public void setItem(List<BondedStockMerchantOrderItem> item) {
        this.item = item;
    }

    public String getPlatSn() {
        return platSn;
    }

    public void setPlatSn(String platSn) {
        this.platSn = platSn;
    }

    public String getMerchName() {
        return merchName;
    }

    public void setMerchName(String merchName) {
        this.merchName = merchName;
    }

    public String getThirdPartyMerchCode() {
        return thirdPartyMerchCode;
    }

    public void setThirdPartyMerchCode(String thirdPartyMerchCode) {
        this.thirdPartyMerchCode = thirdPartyMerchCode;
    }

    public String getThirdPartyMerchName() {
        return thirdPartyMerchName;
    }

    public void setThirdPartyMerchName(String thirdPartyMerchName) {
        this.thirdPartyMerchName = thirdPartyMerchName;
    }

    public String getShopId() {
        return shopId;
    }

    public void setShopId(String shopId) {
        this.shopId = shopId;
    }

    public String getCusPaymentSucceeded() {
        return cusPaymentSucceeded;
    }

    public void setCusPaymentSucceeded(String cusPaymentSucceeded) {
        this.cusPaymentSucceeded = cusPaymentSucceeded;
    }

    public String getCusOrderSucceeded() {
        return cusOrderSucceeded;
    }

    public void setCusOrderSucceeded(String cusOrderSucceeded) {
        this.cusOrderSucceeded = cusOrderSucceeded;
    }

    public String getPaySucceeded() {
        return paySucceeded;
    }

    public void setPaySucceeded(String paySucceeded) {
        this.paySucceeded = paySucceeded;
    }

    public String getBuyerPayerCheck() {
        return buyerPayerCheck;
    }

    public void setBuyerPayerCheck(String buyerPayerCheck) {
        this.buyerPayerCheck = buyerPayerCheck;
    }

    public String getExprAgreementType() {
        return exprAgreementType;
    }

    public void setExprAgreementType(String exprAgreementType) {
        this.exprAgreementType = exprAgreementType;
    }

    public String getTransMode() {
        return transMode;
    }

    public void setTransMode(String transMode) {
        this.transMode = transMode;
    }
}
