package com.emato.ccnet.wx.manager.redis.jedis.cluster.config;

import org.apache.commons.pool2.impl.GenericObjectPoolConfig;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import redis.clients.jedis.HostAndPort;
import redis.clients.jedis.JedisCluster;

import java.util.*;

/**
 * @author Scott Chen
 * @since 1.0
 * 2018-01-06 16:14
 */
public class JedisClusterBuilder {
    private static final Logger logger = LoggerFactory.getLogger(JedisClusterBuilder.class);

    //配置文件Redis主机列表
    List<RedisClusterHostList> redisClusterHostList = new LinkedList<>();

    // Jedis cluster 主机节点
    private Set<HostAndPort> jedisClusterHostSet = new LinkedHashSet<>();
    private int connectionTimeout;
    private int soTimeout;
    private int maxAttempts;
    private String password;
    private GenericObjectPoolConfig poolConfig;


    private JedisClusterBuilder() {}

    public static JedisClusterBuilder create() {
        return new JedisClusterBuilder();
    }

    public JedisClusterBuilder buildHost(List<RedisClusterHostList> redisClusterHostList) {
        if (redisClusterHostList == null || redisClusterHostList.isEmpty()) {
            throw new IllegalArgumentException("Redis Cluster Hosts are null, initialize Hosts failed");
        }

        this.redisClusterHostList.addAll(redisClusterHostList);
        buildJedisClusterNode();
        return buildHost(jedisClusterHostSet);
    }

    public JedisClusterBuilder buildHost(Set<HostAndPort> jedisClusterHostSet) {
        this.jedisClusterHostSet.addAll(jedisClusterHostSet);
        return this;
    }

    public JedisCluster build() {
        return new JedisCluster(
                this.jedisClusterHostSet,
                this.connectionTimeout,
                this.soTimeout,
                this.maxAttempts,
                this.password,
                this.poolConfig
        );
    }

    /**
     * 配置文件中主机节点转为Jedis cluster 主机节点
     * @return
     */
    public Set<HostAndPort> buildJedisClusterNode() {
        try {
            redisClusterHostList
                    .stream()
                    .forEach( x -> {
                        jedisClusterHostSet.add(new HostAndPort(x.getHost(), Integer.valueOf(x.getPort())));
                        logger.info("Redis Cluster Host, IP:{}, PORT:{}", x.getHost(), x.getPort());
                    });
        } catch (Exception e) {
            throw new RuntimeException("Redis Cluster Hosts convert error", e);
        }

        if (jedisClusterHostSet == null || jedisClusterHostSet.isEmpty()) {
            return Collections.emptySet();
        }
        return jedisClusterHostSet;
    }

    public JedisClusterBuilder setJedisClusterHostSet(Set<HostAndPort> jedisClusterHostSet) {
        this.jedisClusterHostSet = jedisClusterHostSet;
        return this;
    }

    public JedisClusterBuilder setConnectionTimeout(int connectionTimeout) {
        this.connectionTimeout = connectionTimeout;
        return this;
    }

    public JedisClusterBuilder setSoTimeout(int soTimeout) {
        this.soTimeout = soTimeout;
        return this;
    }

    public JedisClusterBuilder setMaxAttempts(int maxAttempts) {
        this.maxAttempts = maxAttempts;
        return this;
    }

    public JedisClusterBuilder setPassword(String password) {
        this.password = password;
        return this;
    }

    public JedisClusterBuilder setPoolConfig(GenericObjectPoolConfig poolConfig) {
        this.poolConfig = poolConfig;
        return this;
    }



}
