package com.emato.ccnet.wx.rabbitMq;

import org.mariadb.jdbc.internal.logging.Logger;
import org.mariadb.jdbc.internal.logging.LoggerFactory;
import org.springframework.amqp.core.Binding;
import org.springframework.amqp.core.BindingBuilder;
import org.springframework.amqp.core.DirectExchange;
import org.springframework.amqp.core.Queue;
import org.springframework.amqp.rabbit.connection.CachingConnectionFactory;
import org.springframework.amqp.rabbit.connection.ConnectionFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.core.env.Environment;

/**
 * rabbitmq 的配置类
 */
public class RabbitMqConfig {
    public static final Logger logger = LoggerFactory.getLogger(RabbitMqConfig.class);

    @Autowired
    private Environment env;
    /************************************   mq配置默认信息    ************************************/
    /**
     * 消息交换机的名字
     */
//    public static final String EXCHANGE = "OMS_MQ_EXCHANGE";
    public static final String EXCHANGE = "e.normal.direct.oms.csp.exchange";

    /**
     * 配置链接信息
     *
     * @return
     */
    @Bean
    public ConnectionFactory connectionFactory() {
        CachingConnectionFactory connectionFactory = new CachingConnectionFactory();
        connectionFactory.setAddresses(env.getProperty("spring.rabbitmq.host")+":"+env.getProperty("spring.rabbitmq.port"));
        connectionFactory.setUsername(env.getProperty("spring.rabbitmq.username"));
        connectionFactory.setPassword(env.getProperty("spring.rabbitmq.password"));
        connectionFactory.setVirtualHost(env.getProperty("spring.rabbitmq.virtual-host"));
        connectionFactory.setPublisherConfirms(true); // 必须要设置
        return connectionFactory;
    }

    /**
     * 配置消息交换机
     * 针对消费者配置
     * FanoutExchange: 将消息分发到所有的绑定队列，无routingkey的概念
     * HeadersExchange ：通过添加属性key-value匹配
     * DirectExchange:按照routingkey分发到指定队列
     * TopicExchange:多关键字匹配
     */
    @Bean
    public DirectExchange defaultExchange() {
        return new DirectExchange(EXCHANGE, true, false);
    }

//    //ccnet获取csp海关回执数据队列key
//    public static final String CSP_KEY_RECEIVE_MSG_SEND_OMS_ORDER_CEB311 = "CSP_KEY_RECEIVE_MSG_SEND_OMS_ORDER_CEB311";
//    //ccnet获取csp海关回执数据队列名称
//    public static final String CSP_QUEUE_RECEIVE_MSG_SEND_OMS_ORDER_CEB311 = "CSP_QUEUE_RECEIVE_MSG_SEND_OMS_ORDER_CEB311";

    //ccnet获取csp海关回执数据队列key
    public static final String CSP_KEY_RECEIVE_MSG_SEND_OMS_ORDER_CEB311 = "k.normal.csp.order.to.zw.order.response";
    //ccnet获取csp海关回执数据队列名称
    public static final String CSP_QUEUE_RECEIVE_MSG_SEND_OMS_ORDER_CEB311 = "q.normal.csp.order.to.zw.order.response";

    /**
     * 配置ccnet获取csp海关回执成功或申报失败消息队列
     * 针对消费者配置
     *
     * @return
     */
    @Bean
    public Queue ccnetGainDeclareCspQueue() {
        return new Queue(RabbitMqConfig.CSP_QUEUE_RECEIVE_MSG_SEND_OMS_ORDER_CEB311, true); //队列持久

    }

    /**
     * 将海关回执消息队列与交换机绑定
     * 针对消费者配置
     *
     * @return
     */
    @Bean
    public Binding ccnetGainDeclareCspQueueBinding() {
        return BindingBuilder.bind(this.ccnetGainDeclareCspQueue()).to(defaultExchange()).with(RabbitMqConfig.CSP_KEY_RECEIVE_MSG_SEND_OMS_ORDER_CEB311);
    }

}
