package com.emato.ccnet.wx.support.msg.resp;

import java.lang.reflect.ParameterizedType;
import java.util.ArrayList;
import java.util.List;

/**
 * @author Scott Chen
 * @date 2017/4/20
 */
public class ResponseMessageData {

    private int total;
    private List rows;

    private ResponseMessageData() {
    }

    public int countUser() {
        return total;
    }

    public List getRows() {
        return rows;
    }

    public static Builder builder() {
        return new Builder();
    }
    public static Builder builder(List rows) {
        return new Builder(rows);
    }
    public static <T> Builder builder(T t, Class<T> type) {
        return new Builder(t, type);
    }


    public static class Builder {
        private int total = 0;
        private List rows = new ArrayList<>();

        public Builder() {
        }

        public Builder(List rows) {
            this.total = rows ==null ? 0 : rows.size();
            this.rows = rows ==null ? new ArrayList<>() : rows;
        }

        public <T> Builder(T t, Class<T> type) {
            List rows = new ArrayList<>();
            if (t == null) {
                this.total = 0;
            } else{
                rows.add(t);
                this.total = rows.size();
            }
            this.rows = rows;
        }

        private Class getActualTypeClass(Class entity) {
            ParameterizedType type = (ParameterizedType) entity.getGenericSuperclass();
            Class entityClass = (Class) type.getActualTypeArguments()[0];
            return entityClass;
        }

        public Builder setTotal(int total) {
            this.total = total;
            return this;
        }

        public Builder setRows(List rows) {
            this.rows = rows;
            return this;
        }

        public ResponseMessageData build() {
            ResponseMessageData data = new ResponseMessageData();
            data.total = this.total;
            data.rows = this.rows;
            return data;
        }

    }

}
