package com.emato.ccnet.wx.util;

import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;

/**
 * @author Scott Chen
 * @version 1.0
 * 2017-11-07 23:14
 */
public class FileUtil {


    //------------------------------ readFile ------------------------------

    /**
     * 读取文件为字符串
     * @param filePath
     * @return
     */
    public static String readFile(String filePath) {
        if (StringUtils.isBlank(filePath)) {
            return null;
        }

        String str = "";

        try {
            File file = new File(filePath);
            if (!file.exists()) {
                return null;
            }

            FileInputStream in = new FileInputStream(file);

            // size 为字串的长度 ，这里一次性读完
            int size = in.available();
            byte[] buffer = new byte[size];
            in.read(buffer);
            in.close();

            str = new String(buffer, "utf-8");

        } catch (IOException e) {
            return null;
        }
        return str;
    }

    public static String readFile(String fileDir, String fileName) {
        if (StringUtils.isBlank(fileDir)) {
            return readFile(fileName);
        }

        if (StringUtils.isBlank(fileName)) {
            return null;
        }

        String str = "";

        String filePath;
        try {

            File dir = new File(fileDir);
            if (!dir.exists()) {
                return null;
            }

            if (!fileDir.endsWith("/") && !fileName.startsWith("/")) {
                filePath = fileDir + "/" + fileName;
            } else if(fileDir.endsWith("/") && fileName.startsWith("/")) {
                filePath = fileDir + fileName.substring(1, fileName.length());
            }else {
                filePath = fileDir + fileName;
            }

            File file = new File(filePath);
            if (!file.exists()) {
                return null;
            }

            FileInputStream in = new FileInputStream(file);

            // size 为字串的长度 ，这里一次性读完
            int size = in.available();
            byte[] buffer = new byte[size];
            in.read(buffer);
            in.close();

            str = new String(buffer, "utf-8");

        } catch (IOException e) {
            return null;
        }
        return str;
    }


    //------------------------------ outFile ------------------------------

    public static boolean outFile(String str, String filePath) {
        if (StringUtils.isBlank(filePath)) {
            return false;
        }

        FileOutputStream fop;

        try {
            File file = new File(filePath);
            if (!file.exists()) {
                file.createNewFile();
            }
            fop = new FileOutputStream(file);

            byte[] bytes = str.getBytes();
            fop.write(bytes);
            fop.flush();
            fop.close();
        } catch (IOException e) {
            return false;
        }
        return true;
    }


    public static boolean outFile(String str, String fileDir, String fileName) {
        if (StringUtils.isBlank(fileDir)) {
            return outFile(str, fileName);
        }

        FileOutputStream fop;

        String filePath;
        try {
            File dir = new File(fileDir);
            if (!dir.exists()) {
                dir.mkdirs();
            }

            if (!fileDir.endsWith("/") && !fileName.startsWith("/")) {
                filePath = fileDir + "/" + fileName;
            } else if(fileDir.endsWith("/") && fileName.startsWith("/")) {
                filePath = fileDir + fileName.substring(1, fileName.length());
            }else {
                filePath = fileDir + fileName;
            }

            File file = new File(filePath);
            if (!file.exists()) {
                file.createNewFile();
            }

            fop = new FileOutputStream(file);

            byte[] bytes = str.getBytes();
            fop.write(bytes);
            fop.flush();
            fop.close();
        } catch (IOException e) {
            return false;
        }
        return true;
    }

    //------------------------------ writeFile ------------------------------

    public static boolean writeFile(String str, String filePath) {
        if (StringUtils.isBlank(filePath)) {
            return false;
        }

        try {
            File file = new File(filePath);
            if (!file.exists()) {
                file.createNewFile();
            }

            FileUtils.writeByteArrayToFile(file, str.getBytes());

        } catch (IOException e) {
            return false;
        }
        return true;
    }

    public static boolean writeFile(String str, String fileDir, String fileName) {
        if (StringUtils.isBlank(fileDir)) {
            return writeFile(str, fileName);
        }

        if (StringUtils.isBlank(fileName)) {
            return false;
        }

        String filePath;
        try {
            File dir = new File(fileDir);
            if (!dir.exists()) {
                dir.mkdirs();
            }

            if (!fileDir.endsWith("/") && !fileName.startsWith("/")) {
                filePath = fileDir + "/" + fileName;
            } else if(fileDir.endsWith("/") && fileName.startsWith("/")) {
                filePath = fileDir + fileName.substring(1, fileName.length());
            }else {
                filePath = fileDir + fileName;
            }


            File file = new File(filePath);
            if (!file.exists()) {
                file.createNewFile();
            }

            FileUtils.writeByteArrayToFile(file, str.getBytes());

        } catch (IOException e) {
            return false;
        }
        return true;

    }

}
