package com.emato.ccnet.wx.util;

import org.apache.commons.codec.digest.DigestUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.UnsupportedEncodingException;
import java.security.SignatureException;
import java.util.HashMap;
import java.util.Map;
import java.util.TreeMap;

/**
 * @author Scott Chen
 * @since 1.0
 * 2018-03-10
 */
public class MD5Sign {

    private final static Logger logger = LoggerFactory.getLogger(MD5Sign.class);

    private static String INPUT_CHARSET_UTF8 = "UTF-8";

    /**
     * Map按Key排序
     * @param params
     * @return
     */
    public static Map<String, Object> sortMapByKey(Map<String, Object> params) {
        if (params == null || params.isEmpty()) {
            return null;
        }

        Map<String, Object> sortMap = new TreeMap<String, Object>(
                (x, y) -> x.compareTo(y)
        );

        sortMap.putAll(params);
        return sortMap;
    }

    /**
     * Map 的Key和Value连接
     * @param params
     * @return
     */
    public static String buildParamsStr(Map<String, Object> params) {
        if (params == null || params.isEmpty()) {
            return null;
        }
        StringBuffer sb = new StringBuffer();
        for (Map.Entry entry : params.entrySet()) {
            sb.append(String.valueOf(entry.getKey()) + String.valueOf(entry.getValue()));
        }
        return sb.toString();
    }

    /**
     * 请求参数前添加密钥
     * @param text
     * @param secretKey
     * @return
     */
    public static String insertSecretKey(String text, String secretKey) {
        return new StringBuffer(text).insert(0, secretKey).toString();
    }

    /**
     * 签名
     * @param params 请求参数
     * @param secretKey 密钥
     * @return
     */
    public static String sign(Map<String, Object> params, String secretKey) {
        String paramsStr = insertSecretKey(buildParamsStr(sortMapByKey(params)), secretKey);
        String signature = DigestUtils.md5Hex(getContentBytes(paramsStr, INPUT_CHARSET_UTF8));
        logger.debug("加签: {}", signature);
        return signature;
    }

    /**
     * 验证签名
     * @param signature 签名
     * @param params 请求参数
     * @param secretKey 密钥
     * @return
     */
    public static boolean verify(String signature, Map<String, Object> params, String secretKey) {
        String paramsStr = insertSecretKey(buildParamsStr(sortMapByKey(params)), secretKey);
        String calculateSign = DigestUtils.md5Hex(getContentBytes(paramsStr, INPUT_CHARSET_UTF8));
        boolean verify =  calculateSign.equals(signature) ? true : false;
        logger.debug("系统生成的签名：:{}",calculateSign);
        logger.debug("验签: {}", signature);
        logger.debug("验签结果: {}", verify);
        return verify;
    }


    /**
     * @param content
     * @param charset
     * @return
     * @throws SignatureException
     * @throws UnsupportedEncodingException
     */
    private static byte[] getContentBytes(String content, String charset) {
        if (charset == null || "".equals(charset)) {
            return content.getBytes();
        }
        try {
            return content.getBytes(charset);
        } catch (UnsupportedEncodingException e) {
            throw new RuntimeException("MD5签名过程中出现错误,指定的编码集不对,您目前指定的编码集是:" + charset);
        }
    }



    public static void main(String[] args) {
        Map map = new HashMap();
        //门店商品申请
     //  String entity ="{\"merchId\": \"mhbs982829133821927424\",\"shopId\": \"wbsp988615718893137920\",\"thirdPartyMerchCode\":\"\", \"sinGoodsDocSn\": \"2018042413361111111\",\"consigneeName\": \"赵门商\", \"consigneeProvinceCode\": \"220000\", \"consigneeProvince\": \"吉林省\", \"consigneeCityCode\": \"630200\", \"consigneeCity\": \"吉林市\", \"consigneeDistrictCode\": \"340503\",\"consigneeDistrict\": \"龙潭区\",\"consigneeTel\":\"13558390776\", \"consigneeCountryCode\": \"142\",\"consigneeAddress\": \"西丽镇大学城\", \"billTime\": \"2018-03-01 03:49:08\", \"item\": [{ \"sku\": \"ISZWKJ004907\", \"sellQty\": 3}]}";
      // String entity ="{\"merchId\": \"mhbs982829133821927424\",\"shopId\": \"wbsp988615718893137920\",\"thirdPartyMerchCode\":\"\", \"sinGoodsDocSn\": \"2018042413362222222\",\"consigneeName\": \"赵门商2\", \"consigneeProvinceCode\": \"220000\", \"consigneeProvince\": \"吉林省\", \"consigneeCityCode\": \"630200\", \"consigneeCity\": \"吉林市\", \"consigneeDistrictCode\": \"340503\",\"consigneeDistrict\": \"龙潭区\",\"consigneeTel\":\"13558390776\", \"consigneeCountryCode\": \"142\",\"consigneeAddress\": \"西丽镇大学城\", \"billTime\": \"2018-03-01 03:49:08\", \"item\": [{ \"sku\": \"ISZWKJ004914\", \"sellQty\": 3}]}";
      //  String entity ="{\"merchId\": \"mhbs982829133821927424\",\"shopId\": \"wbsp988615718893137920\",\"thirdPartyMerchCode\":\"\", \"sinGoodsDocSn\": \"2018042413363333333\",\"consigneeName\": \"赵门商3\", \"consigneeProvinceCode\": \"220000\", \"consigneeProvince\": \"吉林省\", \"consigneeCityCode\": \"630200\", \"consigneeCity\": \"吉林市\", \"consigneeDistrictCode\": \"340503\",\"consigneeDistrict\": \"龙潭区\",\"consigneeTel\":\"13558390776\", \"consigneeCountryCode\": \"142\",\"consigneeAddress\": \"西丽镇大学城\", \"billTime\": \"2018-03-01 03:49:08\", \"item\": [{ \"sku\": \"ISZWKJ004908\", \"sellQty\": 3}]}";
//        String entity ="{\"merchId\": \"mhbs982829133821927424\",\"shopId\": \"wbsp988615718893137920\",\"thirdPartyMerchCode\":\"\", \"sinGoodsDocSn\": \"2018042413364444444\",\"consigneeName\": \"赵门商4\", \"consigneeProvinceCode\": \"220000\", \"consigneeProvince\": \"吉林省\", \"consigneeCityCode\": \"630200\", \"consigneeCity\": \"吉林市\", \"consigneeDistrictCode\": \"340503\",\"consigneeDistrict\": \"龙潭区\",\"consigneeTel\":\"13558390776\", \"consigneeCountryCode\": \"142\",\"consigneeAddress\": \"西丽镇大学城\", \"billTime\": \"2018-03-01 03:49:08\", \"item\": [{ \"sku\": \"ISZWKJ004915\", \"sellQty\": 3}]}";
//        String entity ="{\"merchId\": \"mhbs982829133821927424\",\"shopId\": \"wbsp988615718893137920\",\"thirdPartyMerchCode\":\"\", \"sinGoodsDocSn\": \"2018042413365555555\",\"consigneeName\": \"赵门商5\", \"consigneeProvinceCode\": \"220000\", \"consigneeProvince\": \"吉林省\", \"consigneeCityCode\": \"630200\", \"consigneeCity\": \"吉林市\", \"consigneeDistrictCode\": \"340503\",\"consigneeDistrict\": \"龙潭区\",\"consigneeTel\":\"13558390776\", \"consigneeCountryCode\": \"142\",\"consigneeAddress\": \"西丽镇大学城\", \"billTime\": \"2018-03-01 03:49:08\", \"item\": [{ \"sku\": \"ISZWKJ004910\", \"sellQty\": 3}]}";
//        String entity ="{\"merchId\": \"mhbs982829133821927424\",\"shopId\": \"wbsp988615718893137920\",\"thirdPartyMerchCode\":\"\", \"sinGoodsDocSn\": \"2018042413366666666\",\"consigneeName\": \"赵门商6\", \"consigneeProvinceCode\": \"220000\", \"consigneeProvince\": \"吉林省\", \"consigneeCityCode\": \"630200\", \"consigneeCity\": \"吉林市\", \"consigneeDistrictCode\": \"340503\",\"consigneeDistrict\": \"龙潭区\",\"consigneeTel\":\"13558390776\", \"consigneeCountryCode\": \"142\",\"consigneeAddress\": \"西丽镇大学城\", \"billTime\": \"2018-03-01 03:49:08\", \"item\": [{ \"sku\": \"ISZWKJ004916\", \"sellQty\": 3}]}";
//        String entity ="{\"merchId\": \"mhbs982829133821927424\",\"shopId\": \"wbsp988615718893137920\",\"thirdPartyMerchCode\":\"\", \"sinGoodsDocSn\": \"2018042413367777777\",\"consigneeName\": \"赵门商7\", \"consigneeProvinceCode\": \"220000\", \"consigneeProvince\": \"吉林省\", \"consigneeCityCode\": \"630200\", \"consigneeCity\": \"吉林市\", \"consigneeDistrictCode\": \"340503\",\"consigneeDistrict\": \"龙潭区\",\"consigneeTel\":\"13558390776\", \"consigneeCountryCode\": \"142\",\"consigneeAddress\": \"西丽镇大学城\", \"billTime\": \"2018-03-01 03:49:08\", \"item\": [{ \"sku\": \"ISZWKJ004911\", \"sellQty\": 3}]}";
//        String entity ="{\"merchId\": \"mhbs982829133821927424\",\"shopId\": \"wbsp988615718893137920\",\"thirdPartyMerchCode\":\"\", \"sinGoodsDocSn\": \"2018042413368888888\",\"consigneeName\": \"赵门商8\", \"consigneeProvinceCode\": \"220000\", \"consigneeProvince\": \"吉林省\", \"consigneeCityCode\": \"630200\", \"consigneeCity\": \"吉林市\", \"consigneeDistrictCode\": \"340503\",\"consigneeDistrict\": \"龙潭区\",\"consigneeTel\":\"13558390776\", \"consigneeCountryCode\": \"142\",\"consigneeAddress\": \"西丽镇大学城\", \"billTime\": \"2018-03-01 03:49:08\", \"item\": [{ \"sku\": \"ISZWKJ004917\", \"sellQty\": 3}]}";
//        String entity ="{\"merchId\": \"mhbs982829133821927424\",\"shopId\": \"wbsp988615718893137920\",\"thirdPartyMerchCode\":\"\", \"sinGoodsDocSn\": \"2018042413369999999\",\"consigneeName\": \"赵门商9\", \"consigneeProvinceCode\": \"220000\", \"consigneeProvince\": \"吉林省\", \"consigneeCityCode\": \"630200\", \"consigneeCity\": \"吉林市\", \"consigneeDistrictCode\": \"340503\",\"consigneeDistrict\": \"龙潭区\",\"consigneeTel\":\"13558390776\", \"consigneeCountryCode\": \"142\",\"consigneeAddress\": \"西丽镇大学城\", \"billTime\": \"2018-03-01 03:49:08\", \"item\": [{ \"sku\": \"ISZWKJ004918\", \"sellQty\": 3}]}";
//        String entity ="{\"merchId\": \"mhbs982829133821927424\",\"shopId\": \"wbsp988615718893137920\",\"thirdPartyMerchCode\":\"\", \"sinGoodsDocSn\": \"2018042413360000000\",\"consigneeName\": \"赵门商0\", \"consigneeProvinceCode\": \"220000\", \"consigneeProvince\": \"吉林省\", \"consigneeCityCode\": \"630200\", \"consigneeCity\": \"吉林市\", \"consigneeDistrictCode\": \"340503\",\"consigneeDistrict\": \"龙潭区\",\"consigneeTel\":\"13558390776\", \"consigneeCountryCode\": \"142\",\"consigneeAddress\": \"西丽镇大学城\", \"billTime\": \"2018-03-01 03:49:08\", \"item\": [{ \"sku\": \"ISZWKJ004912\", \"sellQty\": 3}]}";

        //门店销售
      // String entity ="{\"shopId\":\"wbsp988615718893137920\",\"merchId\": \"mhbs982829133821927424\",\"thirdPartyMerchCode\":\"\",\"thirdPartyMerchName\":\"\",\"merchName\":null, \"merchOrderId\": \"20180424144598111111\",\"buyerIdType\": \"1\", \"buyerIdCode\": \"430611199211011111\", \"buyerName\": \"刘门销\", \"buyerTel\": \"15814451534\", \"payerIdType\": \"1\", \"payerIdCode\": \"430611199211011111\", \"payerName\": \"刘销\",\"payerMob\": \"15814451534\", \"consigneeIdType\": \"1\", \"consigneeIdCode\": \"430611199211011111\", \"consigneeName\": \"刘销\", \"consigneeMob\": \"15814451534\", \"consigneeTel\": \"0792-3530222\", \"consigneeCountryCode\": \"142\", \"consigneeProvinceCode\": \"330000\", \"consigneeProvince\": \"江西省\",\"consigneeCityCode\": \"630200\", \"consigneeCity\": \"九江市\",\"consigneeDistrictCode\": \"340503\",\"consigneeDistrict\": \"武宁区\", \"consigneeAddress\": \"武宁区第三大道馆\", \"consigneeZipCode\": \"1515\", \"payEntCusCode\": \"4403160Z3Y\", \"payNo\": \"2017111521001104000249622000\", \"acturalPaid\": 118.62000, \"payTime\": \"2017-11-15 03:15:00\", \"exprType\": \"00\", \"exprCompId\": \"yunda\", \"buyerBillTime\": \"2018-03-01 03:49:08\", \"declExprFee\": 0.00, \"declPostTax\": 0.00, \"extraTag\": \"ZWSH418736358190878720\", \"remark\": null, \"platSn\": null, \"item\": [{ \"sku\": \"ISZWKJ004907\", \"sellUnitPrice\": 34.23000, \"sellQty\": 12}] }";
        //String entity ="{\"shopId\":\"wbsp988615718893137920\",\"merchId\": \"mhbs982829133821927424\",\"thirdPartyMerchCode\":\"\",\"thirdPartyMerchName\":\"\",\"merchName\":null, \"merchOrderId\": \"20180424144598222222\",\"buyerIdType\": \"1\", \"buyerIdCode\": \"430611199211011111\", \"buyerName\": \"刘销2\", \"buyerTel\": \"15814451534\", \"payerIdType\": \"1\", \"payerIdCode\": \"430611199211011111\", \"payerName\": \"刘销2\",\"payerMob\": \"15814451534\", \"consigneeIdType\": \"1\", \"consigneeIdCode\": \"430611199211011111\", \"consigneeName\": \"刘销\", \"consigneeMob\": \"15814451534\", \"consigneeTel\": \"0792-3530222\", \"consigneeCountryCode\": \"142\", \"consigneeProvinceCode\": \"330000\", \"consigneeProvince\": \"江西省\",\"consigneeCityCode\": \"630200\", \"consigneeCity\": \"九江市\",\"consigneeDistrictCode\": \"340503\",\"consigneeDistrict\": \"武宁区\", \"consigneeAddress\": \"武宁区第三大道馆\", \"consigneeZipCode\": \"1515\", \"payEntCusCode\": \"4403160Z3Y\", \"payNo\": \"2017111521001104000249622000\", \"acturalPaid\": 118.62000, \"payTime\": \"2017-11-15 03:15:00\", \"exprType\": \"00\", \"exprCompId\": \"yunda\", \"buyerBillTime\": \"2018-03-01 03:49:08\", \"declExprFee\": 0.00, \"declPostTax\": 0.00, \"extraTag\": \"ZWSH418736358190878720\", \"remark\": null, \"platSn\": null, \"item\": [{ \"sku\": \"ISZWKJ004914\", \"sellUnitPrice\": 34.23000, \"sellQty\": 12}] }";
        //String entity ="{\"shopId\":\"wbsp988615718893137920\",\"merchId\": \"mhbs982829133821927424\",\"thirdPartyMerchCode\":\"\",\"thirdPartyMerchName\":\"\",\"merchName\":null, \"merchOrderId\": \"20180424144598333333\",\"buyerIdType\": \"1\", \"buyerIdCode\": \"430611199211011111\", \"buyerName\": \"刘销3\", \"buyerTel\": \"15814451534\", \"payerIdType\": \"1\", \"payerIdCode\": \"430611199211011111\", \"payerName\": \"刘销3\",\"payerMob\": \"15814451534\", \"consigneeIdType\": \"1\", \"consigneeIdCode\": \"430611199211011111\", \"consigneeName\": \"刘销\", \"consigneeMob\": \"15814451534\", \"consigneeTel\": \"0792-3530222\", \"consigneeCountryCode\": \"142\", \"consigneeProvinceCode\": \"330000\", \"consigneeProvince\": \"江西省\",\"consigneeCityCode\": \"630200\", \"consigneeCity\": \"九江市\",\"consigneeDistrictCode\": \"340503\",\"consigneeDistrict\": \"武宁区\", \"consigneeAddress\": \"武宁区第三大道馆\", \"consigneeZipCode\": \"1515\", \"payEntCusCode\": \"4403160Z3Y\", \"payNo\": \"2017111521001104000249622000\", \"acturalPaid\": 118.62000, \"payTime\": \"2017-11-15 03:15:00\", \"exprType\": \"00\", \"exprCompId\": \"yunda\", \"buyerBillTime\": \"2018-03-01 03:49:08\", \"declExprFee\": 0.00, \"declPostTax\": 0.00, \"extraTag\": \"ZWSH418736358190878720\", \"remark\": null, \"platSn\": null, \"item\": [{ \"sku\": \"ISZWKJ004908\", \"sellUnitPrice\": 34.23000, \"sellQty\": 12}] }";
       // String entity ="{\"shopId\":\"wbsp988615718893137920\",\"merchId\": \"mhbs982829133821927424\",\"thirdPartyMerchCode\":\"\",\"thirdPartyMerchName\":\"\",\"merchName\":null, \"merchOrderId\": \"20180424144598444444\",\"buyerIdType\": \"1\", \"buyerIdCode\": \"430611199211011111\", \"buyerName\": \"刘销4\", \"buyerTel\": \"15814451534\", \"payerIdType\": \"1\", \"payerIdCode\": \"430611199211011111\", \"payerName\": \"刘销4\",\"payerMob\": \"15814451534\", \"consigneeIdType\": \"1\", \"consigneeIdCode\": \"430611199211011111\", \"consigneeName\": \"刘销\", \"consigneeMob\": \"15814451534\", \"consigneeTel\": \"0792-3530222\", \"consigneeCountryCode\": \"142\", \"consigneeProvinceCode\": \"330000\", \"consigneeProvince\": \"江西省\",\"consigneeCityCode\": \"630200\", \"consigneeCity\": \"九江市\",\"consigneeDistrictCode\": \"340503\",\"consigneeDistrict\": \"武宁区\", \"consigneeAddress\": \"武宁区第三大道馆\", \"consigneeZipCode\": \"1515\", \"payEntCusCode\": \"4403160Z3Y\", \"payNo\": \"2017111521001104000249622000\", \"acturalPaid\": 118.62000, \"payTime\": \"2017-11-15 03:15:00\", \"exprType\": \"00\", \"exprCompId\": \"yunda\", \"buyerBillTime\": \"2018-03-01 03:49:08\", \"declExprFee\": 0.00, \"declPostTax\": 0.00, \"extraTag\": \"ZWSH418736358190878720\", \"remark\": null, \"platSn\": null, \"item\": [{ \"sku\": \"ISZWKJ004915\", \"sellUnitPrice\": 34.23000, \"sellQty\": 12}] }";


        //保税展示
        //String entity ="{ \"merchId\": \"mhbs982829133821927424\",\"merchName\": 天虹, \"merchOrderId\": \"20180426144917111111\",\"buyerIdType\": \"1\", \"buyerIdCode\": \"430611199211055555\", \"buyerName\": \"何展\", \t\"buyerTel\": \"15814459008\", \"payerIdType\": \"1\", \"payerIdCode\": \"430611199211055555\", \"payerName\": \"何展\",\"payerMob\": \"15814459008\", \"consigneeIdType\": \"1\", \"consigneeIdCode\": \"430611199211055555\", \"consigneeName\": \"何展\", \"consigneeMob\": \"15814459008\", \"consigneeTel\": \"15886451515\", \"consigneeCountryCode\": \"142\", \"consigneeProvinceCode\": \"220000\", \"consigneeProvince\": \"吉林省\",\"consigneeCityCode\": \"630200\", \"consigneeCity\": \"长春市\",\"consigneeDistrictCode\": \"340503\",\"consigneeDistrict\": \"南关区\", \"consigneeAddress\": \"南岭街道\", \"consigneeZipCode\": \"111111\", \"payEntCusCode\": \"4403160Z3Y\", \"payNo\": \"2017111521001104000249633333\", \"acturalPaid\": 118.62000, \"payTime\": \"2017-11-15 03:15:00\", \"exprType\": \"00\", \"exprCompId\": \"yunda\", \"buyerBillTime\": \"2018-03-01 03:49:08\", \"declExprFee\": 0.00, \"declPostTax\": 0.00, \"voyageNo\":\"1\",\"billNo\":\"111\",\"startCountryCode\":\"1234\",\"extraTag\": \"ZWSH418736358190878720\", \"shopId\":\"wbsp988615718893137920\",\"remark\": null, \"platSn\": null, \"item\": [{ \"sku\": \"ISZWKJ004907\", \"sellUnitPrice\": 34.23000, \"sellQty\": 7}]}";
        //String entity ="{ \"merchId\": \"mhbs982829133821927424\",\"merchName\": 天虹, \"merchOrderId\": \"20180426144917222222\",\"buyerIdType\": \"1\", \"buyerIdCode\": \"430611199211055555\", \"buyerName\": \"何展2\", \t\"buyerTel\": \"15814459008\", \"payerIdType\": \"1\", \"payerIdCode\": \"430611199211055555\", \"payerName\": \"何展2\",\"payerMob\": \"15814459008\", \"consigneeIdType\": \"1\", \"consigneeIdCode\": \"430611199211055555\", \"consigneeName\": \"何展2\", \"consigneeMob\": \"15814459008\", \"consigneeTel\": \"15886451515\", \"consigneeCountryCode\": \"142\", \"consigneeProvinceCode\": \"220000\", \"consigneeProvince\": \"吉林省\",\"consigneeCityCode\": \"630200\", \"consigneeCity\": \"长春市\",\"consigneeDistrictCode\": \"340503\",\"consigneeDistrict\": \"南关区\", \"consigneeAddress\": \"南岭街道\", \"consigneeZipCode\": \"111111\", \"payEntCusCode\": \"4403160Z3Y\", \"payNo\": \"2017111521001104000249633333\", \"acturalPaid\": 118.62000, \"payTime\": \"2017-11-15 03:15:00\", \"exprType\": \"00\", \"exprCompId\": \"yunda\", \"buyerBillTime\": \"2018-03-01 03:49:08\", \"declExprFee\": 0.00, \"declPostTax\": 0.00, \"voyageNo\":\"1\",\"billNo\":\"111\",\"startCountryCode\":\"1234\",\"extraTag\": \"ZWSH418736358190878720\", \"shopId\":\"wbsp988615718893137920\",\"remark\": null, \"platSn\": null, \"item\": [{ \"sku\": \"ISZWKJ004908\", \"sellUnitPrice\": 34.23000, \"sellQty\": 7}]}";
        //String entity ="{ \"merchId\": \"mhbs982829133821927424\",\"merchName\": 天虹, \"merchOrderId\": \"20180426144917333333\",\"buyerIdType\": \"1\", \"buyerIdCode\": \"430611199211055555\", \"buyerName\": \"何展3\", \t\"buyerTel\": \"15814459008\", \"payerIdType\": \"1\", \"payerIdCode\": \"430611199211055555\", \"payerName\": \"何展3\",\"payerMob\": \"15814459008\", \"consigneeIdType\": \"1\", \"consigneeIdCode\": \"430611199211055555\", \"consigneeName\": \"何展3\", \"consigneeMob\": \"15814459008\", \"consigneeTel\": \"15886451515\", \"consigneeCountryCode\": \"142\", \"consigneeProvinceCode\": \"220000\", \"consigneeProvince\": \"吉林省\",\"consigneeCityCode\": \"630200\", \"consigneeCity\": \"长春市\",\"consigneeDistrictCode\": \"340503\",\"consigneeDistrict\": \"南关区\", \"consigneeAddress\": \"南岭街道\", \"consigneeZipCode\": \"111111\", \"payEntCusCode\": \"4403160Z3Y\", \"payNo\": \"2017111521001104000249633333\", \"acturalPaid\": 118.62000, \"payTime\": \"2017-11-15 03:15:00\", \"exprType\": \"00\", \"exprCompId\": \"yunda\", \"buyerBillTime\": \"2018-03-01 03:49:08\", \"declExprFee\": 0.00, \"declPostTax\": 0.00, \"voyageNo\":\"1\",\"billNo\":\"111\",\"startCountryCode\":\"1234\",\"extraTag\": \"ZWSH418736358190878720\", \"shopId\":\"wbsp988615718893137920\",\"remark\": null, \"platSn\": null, \"item\": [{ \"sku\": \"ISZWKJ004915\", \"sellUnitPrice\": 34.23000, \"sellQty\": 7}]}";
       //## String entity ="{ \"merchId\": \"mhbs982829133821927424\",\"merchName\": 天虹, \"merchOrderId\": \"20180426144917444444\",\"buyerIdType\": \"1\", \"buyerIdCode\": \"430611199211055555\", \"buyerName\": \"何展4\", \t\"buyerTel\": \"15814459008\", \"payerIdType\": \"1\", \"payerIdCode\": \"430611199211055555\", \"payerName\": \"何展4\",\"payerMob\": \"15814459008\", \"consigneeIdType\": \"1\", \"consigneeIdCode\": \"430611199211055555\", \"consigneeName\": \"何展4\", \"consigneeMob\": \"15814459008\", \"consigneeTel\": \"15886451515\", \"consigneeCountryCode\": \"142\", \"consigneeProvinceCode\": \"220000\", \"consigneeProvince\": \"吉林省\",\"consigneeCityCode\": \"630200\", \"consigneeCity\": \"长春市\",\"consigneeDistrictCode\": \"340503\",\"consigneeDistrict\": \"南关区\", \"consigneeAddress\": \"南岭街道\", \"consigneeZipCode\": \"111111\", \"payEntCusCode\": \"4403160Z3Y\", \"payNo\": \"2017111521001104000249633333\", \"acturalPaid\": 118.62000, \"payTime\": \"2017-11-15 03:15:00\", \"exprType\": \"00\", \"exprCompId\": \"yunda\", \"buyerBillTime\": \"2018-03-01 03:49:08\", \"declExprFee\": 0.00, \"declPostTax\": 0.00, \"voyageNo\":\"1\",\"billNo\":\"111\",\"startCountryCode\":\"1234\",\"extraTag\": \"ZWSH418736358190878720\", \"shopId\":\"wbsp988615718893137920\",\"remark\": null, \"platSn\": null, \"item\": [{ \"sku\": \"ISZWKJ004910\", \"sellUnitPrice\": 34.23000, \"sellQty\": 7}]}";
        // String entity ="{ \"merchId\": \"mhbs982829133821927424\",\"merchName\": 天虹, \"merchOrderId\": \"20180426144917555555\",\"buyerIdType\": \"1\", \"buyerIdCode\": \"430611199211055555\", \"buyerName\": \"何展0\", \t\"buyerTel\": \"15814459008\", \"payerIdType\": \"1\", \"payerIdCode\": \"430611199211055555\", \"payerName\": \"何展0\",\"payerMob\": \"15814459008\", \"consigneeIdType\": \"1\", \"consigneeIdCode\": \"430611199211055555\", \"consigneeName\": \"何展0\", \"consigneeMob\": \"15814459008\", \"consigneeTel\": \"15886451515\", \"consigneeCountryCode\": \"142\", \"consigneeProvinceCode\": \"220000\", \"consigneeProvince\": \"吉林省\",\"consigneeCityCode\": \"630200\", \"consigneeCity\": \"长春市\",\"consigneeDistrictCode\": \"340503\",\"consigneeDistrict\": \"南关区\", \"consigneeAddress\": \"南岭街道\", \"consigneeZipCode\": \"111111\", \"payEntCusCode\": \"4403160Z3Y\", \"payNo\": \"2017111521001104000249633333\", \"acturalPaid\": 118.62000, \"payTime\": \"2017-11-15 03:15:00\", \"exprType\": \"00\", \"exprCompId\": \"yunda\", \"buyerBillTime\": \"2018-03-01 03:49:08\", \"declExprFee\": 0.00, \"declPostTax\": 0.00, \"voyageNo\":\"1\",\"billNo\":\"111\",\"startCountryCode\":\"1234\",\"extraTag\": \"ZWSH418736358190878720\", \"shopId\":\"wbsp988615718893137920\",\"remark\": null, \"platSn\": null, \"item\": [{ \"sku\": \"ISZWKJ004913\", \"sellUnitPrice\": 34.23000, \"sellQty\": 7}]}";

        //保税备货
       // String entity ="{\"merchId\": \"mhbs982829133821927424\",\"thirdPartyMerchCode\":\"\",\"thirdPartyMerchName\":\"\",\"merchName\": null, \"merchOrderId\": \"2018042614541111111\",\"buyerIdType\": \"1\", \"buyerIdCode\": \"430611199211081111\", \"buyerName\": \"王备\", \t\"buyerTel\": \"15814451534\", \"payerIdType\": \"1\", \"payerIdCode\": \"430611199211081111\", \"payerName\": \"王备\",\"payerMob\": \"15814451534\", \"consigneeIdType\": \"1\", \"consigneeIdCode\": \"430611199211081111\", \"consigneeName\": \"王备\", \"consigneeMob\": \"15814451534\", \"consigneeTel\": \"15886451515\", \"consigneeCountryCode\": \"142\", \"consigneeProvinceCode\": \"430000\", \"consigneeProvince\": \"湖南省\",\"consigneeCityCode\": \"630200\", \"consigneeCity\": \"怀化市\",\"consigneeDistrictCode\": \"340503\",\"consigneeDistrict\": \"辰溪县\", \"consigneeAddress\": \"洪江区文心六路道馆\", \"consigneeZipCode\": \"1515\", \"payEntCusCode\": \"4403160Z3Y\", \"payNo\": \"2017111521001104000249622093\", \"acturalPaid\": 118.62000, \"payTime\": \"2017-11-15 03:15:00\", \"exprType\": \"00\", \"exprCompId\": \"yunda\", \"buyerBillTime\": \"2018-03-01 03:49:08\", \"declExprFee\": 0.00, \"declPostTax\": 0.00, \"extraTag\": \"ZWSH418736358190878720\", \"remark\": null, \"platSn\": null, \"item\": [{ \"sku\": \"ISZWKJ004907\", \"sellUnitPrice\": 34.23000, \"sellQty\": 9}] }";
       // String entity ="{\"merchId\": \"mhbs982829133821927424\",\"thirdPartyMerchCode\":\"\",\"thirdPartyMerchName\":\"\",\"merchName\": null, \"merchOrderId\": \"2018042614542222222\",\"buyerIdType\": \"1\", \"buyerIdCode\": \"430611199211081111\", \"buyerName\": \"王备2\", \t\"buyerTel\": \"15814451534\", \"payerIdType\": \"1\", \"payerIdCode\": \"430611199211081111\", \"payerName\": \"王备2\",\"payerMob\": \"15814451534\", \"consigneeIdType\": \"1\", \"consigneeIdCode\": \"430611199211081111\", \"consigneeName\": \"王备2\", \"consigneeMob\": \"15814451534\", \"consigneeTel\": \"15886451515\", \"consigneeCountryCode\": \"142\", \"consigneeProvinceCode\": \"430000\", \"consigneeProvince\": \"湖南省\",\"consigneeCityCode\": \"630200\", \"consigneeCity\": \"怀化市\",\"consigneeDistrictCode\": \"340503\",\"consigneeDistrict\": \"辰溪县\", \"consigneeAddress\": \"洪江区文心六路道馆\", \"consigneeZipCode\": \"1515\", \"payEntCusCode\": \"4403160Z3Y\", \"payNo\": \"2017111521001104000249622093\", \"acturalPaid\": 118.62000, \"payTime\": \"2017-11-15 03:15:00\", \"exprType\": \"00\", \"exprCompId\": \"yunda\", \"buyerBillTime\": \"2018-03-01 03:49:08\", \"declExprFee\": 0.00, \"declPostTax\": 0.00, \"extraTag\": \"ZWSH418736358190878720\", \"remark\": null, \"platSn\": null, \"item\": [{ \"sku\": \"ISZWKJ004914\", \"sellUnitPrice\": 34.23000, \"sellQty\": 9}] }";
         String data ="{\"merchId\":\"mhbs996643550862512128\",\"merchName\":\"\",\"thirdPartyMerchCode\":\"\",\"thirdPartyMerchName\":\"\",\"merchOrderId\":\"v100030026\",\"buyerIdType\":1,\"buyerIdCode\":\"232123456543234324\",\"buyerName\":\"啦啦啦\",\"buyerTel\":\"18875954661\",\"payerIdType\":1,\"payerIdCode\":\"232123456543234324\",\"payerName\":\"啦啦啦\",\"payerMob\":\"18875954661\",\"consigneeIdType\":1,\"consigneeIdCode\":\"\",\"consigneeName\":\"啦啦啦\",\"consigneeMob\":\"18875954661\",\"consigneeTel\":\"\",\"consigneeCountryCode\":\"142\",\"consigneeProvinceCode\":\"440000\",\"consigneeProvince\":\"广东省\",\"consigneeCityCode\":\"440300\",\"consigneeCity\":\"深圳市\",\"consigneeDistrictCode\":\"440304\",\"consigneeDistrict\":\"福田区\",\"consigneeAddress\":\"斤斤计较\",\"consigneeZipCode\":\"440304\",\"payEntCusCode\":\"4403160Z3Y\",\"payNo\":\"2017111521001104000249622093\",\"acturalPaid\":349.00,\"payTime\":\"2018-06-07 08:38:32\",\"exprType\":\"00\",\"exprCompId\":\"yunda\",\"buyerBillTime\":\"2018-06-07 08:38:09\",\"declExprFee\":0.0,\"declPostTax\":0.0,\"extraTag\":\"\",\"platSn\":\"\",\"cusPaymentSucceeded\":0,\"cusOrderSucceeded\":0,\"paySucceeded\":\"0\",\"remark\":\"\",\"item\":[{\"sku\":\"ISZWKJ004907\",\"sellUnitPrice\":349.00,\"sellQty\":1}],\"exprAgreementType\":\"00\",\"buyerPayerCheck\":1}";
        // String entity ="{\"merchId\": \"mhbs982829133821927424\",\"thirdPartyMerchCode\":\"\",\"thirdPartyMerchName\":\"\",\"merchName\": null, \"merchOrderId\": \"2018042614544444444\",\"buyerIdType\": \"1\", \"buyerIdCode\": \"430611199211081111\", \"buyerName\": \"王备4\", \t\"buyerTel\": \"15814451534\", \"payerIdType\": \"1\", \"payerIdCode\": \"430611199211081111\", \"payerName\": \"王备4\",\"payerMob\": \"15814451534\", \"consigneeIdType\": \"1\", \"consigneeIdCode\": \"430611199211081111\", \"consigneeName\": \"王备4\", \"consigneeMob\": \"15814451534\", \"consigneeTel\": \"15886451515\", \"consigneeCountryCode\": \"142\", \"consigneeProvinceCode\": \"430000\", \"consigneeProvince\": \"湖南省\",\"consigneeCityCode\": \"630200\", \"consigneeCity\": \"怀化市\",\"consigneeDistrictCode\": \"340503\",\"consigneeDistrict\": \"辰溪县\", \"consigneeAddress\": \"洪江区文心六路道馆\", \"consigneeZipCode\": \"1515\", \"payEntCusCode\": \"4403160Z3Y\", \"payNo\": \"2017111521001104000249622093\", \"acturalPaid\": 118.62000, \"payTime\": \"2017-11-15 03:15:00\", \"exprType\": \"00\", \"exprCompId\": \"yunda\", \"buyerBillTime\": \"2018-03-01 03:49:08\", \"declExprFee\": 0.00, \"declPostTax\": 0.00, \"extraTag\": \"ZWSH418736358190878720\", \"remark\": null, \"platSn\": null, \"item\": [{ \"sku\": \"ISZWKJ004910\", \"sellUnitPrice\": 34.23000, \"sellQty\": 9}] }";
        // String entity ="{"merchId": "mhbs982829133821927424","thirdPartyMerchCode":"","thirdPartyMerchName":"","merchName": null, "merchOrderId": "2018042614545555555","buyerIdType": "1", "buyerIdCode": "430611199211081111", "buyerName": "王备5", 	"buyerTel": "15814451534", "payerIdType": "1", "payerIdCode": "430611199211081111", "payerName": "王备5","payerMob": "15814451534", "consigneeIdType": "1", "consigneeIdCode": "430611199211081111", "consigneeName": "王备5", "consigneeMob": "15814451534", "consigneeTel": "15886451515", "consigneeCountryCode": "142", "consigneeProvinceCode": "430000", "consigneeProvince": "湖南省","consigneeCityCode": "630200", "consigneeCity": "怀化市","consigneeDistrictCode": "340503","consigneeDistrict": "辰溪县", "consigneeAddress": "洪江区文心六路道馆", "consigneeZipCode": "1515", "payEntCusCode": "4403160Z3Y", "payNo": "2017111521001104000249622093", "acturalPaid": 118.62000, "payTime": "2017-11-15 03:15:00", "exprType": "00", "exprCompId": "yunda", "buyerBillTime": "2018-03-01 03:49:08", "declExprFee": 0.00, "declPostTax": 0.00, "extraTag": "ZWSH418736358190878720", "remark": null, "platSn": null, "item": [{ "sku": "ISZWKJ004916", "sellUnitPrice": 34.23000, "sellQty": 9}] }";
        // String entity ="{"merchId": "mhbs982829133821927424","thirdPartyMerchCode":"","thirdPartyMerchName":"","merchName": null, "merchOrderId": "2018046145466666666","buyerIdType": "1", "buyerIdCode": "430611199211081111", "buyerName": "王备6", 	"buyerTel": "15814451534", "payerIdType": "1", "payerIdCode": "430611199211081111", "payerName": "王备6","payerMob": "15814451534", "consigneeIdType": "1", "consigneeIdCode": "430611199211081111", "consigneeName": "王备6", "consigneeMob": "15814451534", "consigneeTel": "15886451515", "consigneeCountryCode": "142", "consigneeProvinceCode": "430000", "consigneeProvince": "湖南省","consigneeCityCode": "630200", "consigneeCity": "怀化市","consigneeDistrictCode": "340503","consigneeDistrict": "辰溪县", "consigneeAddress": "洪江区文心六路道馆", "consigneeZipCode": "1515", "payEntCusCode": "4403160Z3Y", "payNo": "2017111521001104000249622093", "acturalPaid": 118.62000, "payTime": "2017-11-15 03:15:00", "exprType": "00", "exprCompId": "yunda", "buyerBillTime": "2018-03-01 03:49:08", "declExprFee": 0.00, "declPostTax": 0.00, "extraTag": "ZWSH418736358190878720", "remark": null, "platSn": null, "item": [{ "sku": "ISZWKJ004911", "sellUnitPrice": 34.23000, "sellQty": 9}] }";
        // String entity ="{"merchId": "mhbs982829133821927424","thirdPartyMerchCode":"","thirdPartyMerchName":"","merchName": null, "merchOrderId": "2018042614547777777","buyerIdType": "1", "buyerIdCode": "430611199211081111", "buyerName": "王备7", 	"buyerTel": "15814451534", "payerIdType": "1", "payerIdCode": "430611199211081111", "payerName": "王备7","payerMob": "15814451534", "consigneeIdType": "1", "consigneeIdCode": "430611199211081111", "consigneeName": "王备7", "consigneeMob": "15814451534", "consigneeTel": "15886451515", "consigneeCountryCode": "142", "consigneeProvinceCode": "430000", "consigneeProvince": "湖南省","consigneeCityCode": "630200", "consigneeCity": "怀化市","consigneeDistrictCode": "340503","consigneeDistrict": "辰溪县", "consigneeAddress": "洪江区文心六路道馆", "consigneeZipCode": "1515", "payEntCusCode": "4403160Z3Y", "payNo": "2017111521001104000249622093", "acturalPaid": 118.62000, "payTime": "2017-11-15 03:15:00", "exprType": "00", "exprCompId": "yunda", "buyerBillTime": "2018-03-01 03:49:08", "declExprFee": 0.00, "declPostTax": 0.00, "extraTag": "ZWSH418736358190878720", "remark": null, "platSn": null, "item": [{ "sku": "ISZWKJ004917", "sellUnitPrice": 34.23000, "sellQty": 9}] }";
        // String entity ="{"merchId": "mhbs982829133821927424","thirdPartyMerchCode":"","thirdPartyMerchName":"","merchName": null, "merchOrderId": "2018042614548888888","buyerIdType": "1", "buyerIdCode": "430611199211081111", "buyerName": "王备8", 	"buyerTel": "15814451534", "payerIdType": "1", "payerIdCode": "430611199211081111", "payerName": "王备8","payerMob": "15814451534", "consigneeIdType": "1", "consigneeIdCode": "430611199211081111", "consigneeName": "王备8", "consigneeMob": "15814451534", "consigneeTel": "15886451515", "consigneeCountryCode": "142", "consigneeProvinceCode": "430000", "consigneeProvince": "湖南省","consigneeCityCode": "630200", "consigneeCity": "怀化市","consigneeDistrictCode": "340503","consigneeDistrict": "辰溪县", "consigneeAddress": "洪江区文心六路道馆", "consigneeZipCode": "1515", "payEntCusCode": "4403160Z3Y", "payNo": "2017111521001104000249622093", "acturalPaid": 118.62000, "payTime": "2017-11-15 03:15:00", "exprType": "00", "exprCompId": "yunda", "buyerBillTime": "2018-03-01 03:49:08", "declExprFee": 0.00, "declPostTax": 0.00, "extraTag": "ZWSH418736358190878720", "remark": null, "platSn": null, "item": [{ "sku": "ISZWKJ004918", "sellUnitPrice": 34.23000, "sellQty": 9}] }";
        // String entity ="{\"merchId\": \"mhbs982829133821927424\",\"thirdPartyMerchCode\":\"\",\"thirdPartyMerchName\":\"\",\"merchName\": null, \"merchOrderId\": \"2018042614549999999\",\"buyerIdType\": \"1\", \"buyerIdCode\": \"430611199211081111\", \"buyerName\": \"王备9\", \t\"buyerTel\": \"15814451534\", \"payerIdType\": \"1\", \"payerIdCode\": \"430611199211081111\", \"payerName\": \"王备9\",\"payerMob\": \"15814451534\", \"consigneeIdType\": \"1\", \"consigneeIdCode\": \"430611199211081111\", \"consigneeName\": \"王备9\", \"consigneeMob\": \"15814451534\", \"consigneeTel\": \"15886451515\", \"consigneeCountryCode\": \"142\", \"consigneeProvinceCode\": \"430000\", \"consigneeProvince\": \"湖南省\",\"consigneeCityCode\": \"630200\", \"consigneeCity\": \"怀化市\",\"consigneeDistrictCode\": \"340503\",\"consigneeDistrict\": \"辰溪县\", \"consigneeAddress\": \"洪江区文心六路道馆\", \"consigneeZipCode\": \"1515\", \"payEntCusCode\": \"4403160Z3Y\", \"payNo\": \"2017111521001104000249622093\", \"acturalPaid\": 118.62000, \"payTime\": \"2017-11-15 03:15:00\", \"exprType\": \"00\", \"exprCompId\": \"yunda\", \"buyerBillTime\": \"2018-03-01 03:49:08\", \"declExprFee\": 0.00, \"declPostTax\": 0.00, \"extraTag\": \"ZWSH418736358190878720\", \"remark\": null, \"platSn\": null, \"item\": [{ \"sku\": \"ISZWKJ004912\", \"sellUnitPrice\": 34.23000, \"sellQty\": 9}] }";
        // String entity ="{\"merchId\": \"mhbs982829133821927424\",\"thirdPartyMerchCode\":\"\",\"thirdPartyMerchName\":\"\",\"merchName\": null, \"merchOrderId\": \"2018042614540000000\",\"buyerIdType\": \"1\", \"buyerIdCode\": \"430611199211081111\", \"buyerName\": \"王备0\", \t\"buyerTel\": \"15814451534\", \"payerIdType\": \"1\", \"payerIdCode\": \"430611199211081111\", \"payerName\": \"王备0\",\"payerMob\": \"15814451534\", \"consigneeIdType\": \"1\", \"consigneeIdCode\": \"430611199211081111\", \"consigneeName\": \"王备0\", \"consigneeMob\": \"15814451534\", \"consigneeTel\": \"15886451515\", \"consigneeCountryCode\": \"142\", \"consigneeProvinceCode\": \"430000\", \"consigneeProvince\": \"湖南省\",\"consigneeCityCode\": \"630200\", \"consigneeCity\": \"怀化市\",\"consigneeDistrictCode\": \"340503\",\"consigneeDistrict\": \"辰溪县\", \"consigneeAddress\": \"洪江区文心六路道馆\", \"consigneeZipCode\": \"1515\", \"payEntCusCode\": \"4403160Z3Y\", \"payNo\": \"2017111521001104000249622093\", \"acturalPaid\": 118.62000, \"payTime\": \"2017-11-15 03:15:00\", \"exprType\": \"00\", \"exprCompId\": \"yunda\", \"buyerBillTime\": \"2018-03-01 03:49:08\", \"declExprFee\": 0.00, \"declPostTax\": 0.00, \"extraTag\": \"ZWSH418736358190878720\", \"remark\": null, \"platSn\": null, \"item\": [{ \"sku\": \"ISZWKJ004913\", \"sellUnitPrice\": 34.23000, \"sellQty\": 9}] }";
         //String entity ="{\"merchId\": \"mhbs982829133821927424\",\"thirdPartyMerchCode\":\"\",\"thirdPartyMerchName\":\"\",\"merchName\": null, \"merchOrderId\": \"2018042614540000008\",\"buyerIdType\": \"1\", \"buyerIdCode\": \"430611199211081111\", \"buyerName\": \"王备08\", \t\"buyerTel\": \"15814451534\", \"payerIdType\": \"1\", \"payerIdCode\": \"430611199211081111\", \"payerName\": \"王备08\",\"payerMob\": \"15814451534\", \"consigneeIdType\": \"1\", \"consigneeIdCode\": \"430611199211081111\", \"consigneeName\": \"王备08\", \"consigneeMob\": \"15814451534\", \"consigneeTel\": \"15886451515\", \"consigneeCountryCode\": \"142\", \"consigneeProvinceCode\": \"430000\", \"consigneeProvince\": \"湖南省\",\"consigneeCityCode\": \"630200\", \"consigneeCity\": \"怀化市\",\"consigneeDistrictCode\": \"340503\",\"consigneeDistrict\": \"辰溪县\", \"consigneeAddress\": \"洪江区文心六路道馆\", \"consigneeZipCode\": \"1515\", \"payEntCusCode\": \"4403160Z3Y\", \"payNo\": \"2017111521001104000249622093\", \"acturalPaid\": 118.62000, \"payTime\": \"2017-11-15 03:15:00\", \"exprType\": \"00\", \"exprCompId\": \"yunda\", \"buyerBillTime\": \"2018-03-01 03:49:08\", \"declExprFee\": 0.00, \"declPostTax\": 0.00, \"extraTag\": \"ZWSH418736358190878720\", \"remark\": null, \"platSn\": null, \"item\": [{ \"sku\": \"ISZWKJ004908\", \"sellUnitPrice\": 34.23000, \"sellQty\": 9}] }";
        map.put("data", data);
        map.put("timestamp", "1528333470");
        map.put("merchId", "mhbs996643550862512128");

        String si = sign(map, "hzxCjtGlY9d5AJj9");
        verify(si, map, "hzxCjtGlY9d5AJj9");
    }

}
