package com.emato.ccnet.wx.util;

import org.apache.commons.beanutils.BeanUtils;

import java.beans.BeanInfo;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.HashMap;
import java.util.Map;

/**
 * map bean 相互转换
 *
 * @author Scott Chen
 * @version 1.0
 * 2017-09-19 16:58
 */
public class MapBeanUtil {

    /**
     * map 转 bean by BeanUtils
     *
     * @param map
     * @param beanClass
     * @return
     * @throws Exception
     */
    public static Object toObject(Map<String, Object> map, Class<?> beanClass) throws Exception {
        if (map == null) {
            return null;
        }
        Object obj = beanClass.newInstance();
        BeanUtils.populate(obj, map);

        return obj;
    }

    /**
     * bean 转 map   by Introspector
     *
     * @param obj
     * @return
     */
 /*   public static Map<String, Object> fromObject(Object obj) {
        if(obj == null)
            return null;
        return new BeanMap(obj);
    }*/
    public static Map<String, Object> fromObject(Object obj) {

        if (obj == null) {
            return null;
        }

        Map<String, Object> map = new HashMap<String, Object>();
        try {
            Field[] declaredFields = obj.getClass().getDeclaredFields();
            for (Field field : declaredFields) {
                field.setAccessible(true);
                map.put(field.getName(), field.get(obj));
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        return map;
    }

    //----------  使用Introspector进行转换 ----------

    /**
     * map 转 bean by Introspector
     *
     * @param map
     * @param beanClass
     * @return
     * @throws Exception
     */
    public static Object toObjectByIntrospector(Map<String, Object> map, Class<?> beanClass) throws Exception {
        if (map == null)
            return null;

        Object obj = beanClass.newInstance();

        BeanInfo beanInfo = Introspector.getBeanInfo(obj.getClass());
        PropertyDescriptor[] propertyDescriptors = beanInfo.getPropertyDescriptors();
        for (PropertyDescriptor property : propertyDescriptors) {
            Method setter = property.getWriteMethod();
            if (setter != null) {
                setter.invoke(obj, map.get(property.getName()));
            }
        }

        return obj;
    }

    /**
     * bean 转 map   by Introspector
     *
     * @param obj
     * @return
     * @throws Exception
     */
    public static Map<String, Object> fromObjectByIntrospector(Object obj) throws Exception {
        if (obj == null)
            return null;

        Map<String, Object> map = new HashMap<String, Object>();

        BeanInfo beanInfo = Introspector.getBeanInfo(obj.getClass());
        PropertyDescriptor[] propertyDescriptors = beanInfo.getPropertyDescriptors();
        for (PropertyDescriptor property : propertyDescriptors) {
            String key = property.getName();
            if (key.compareToIgnoreCase("class") == 0) {
                continue;
            }
            Method getter = property.getReadMethod();
            Object value = getter != null ? getter.invoke(obj) : null;
            map.put(key, value);
        }

        return map;
    }

    //----------  使用reflect进行转换 ----------

    /**
     * map 转 bean  by Reflect
     *
     * @param map
     * @param beanClass
     * @return
     * @throws Exception
     */
    public static Object toObjectByReflect(Map<String, Object> map, Class<?> beanClass) throws Exception {
        if (map == null)
            return null;

        Object obj = beanClass.newInstance();

        Field[] fields = obj.getClass().getDeclaredFields();
        for (Field field : fields) {
            int mod = field.getModifiers();
            if (Modifier.isStatic(mod) || Modifier.isFinal(mod)) {
                continue;
            }

            field.setAccessible(true);
            field.set(obj, map.get(field.getName()));
        }

        return obj;
    }

    /**
     * bean 转 map   by Reflect
     *
     * @param obj
     * @return
     * @throws Exception
     */
    public static Map<String, Object> fromObjectByReflect(Object obj) throws Exception {
        if (obj == null) {
            return null;
        }

        Map<String, Object> map = new HashMap<String, Object>();

        Field[] declaredFields = obj.getClass().getDeclaredFields();
        for (Field field : declaredFields) {
            field.setAccessible(true);
            map.put(field.getName(), field.get(obj));
        }

        return map;
    }


}
